Package: gena
Type: Package
Title: Genetic Algorithm and Particle Swarm Optimization
Version: 1.0.1
Date: 2026-04-08
Authors@R: c(person("Bogdan", "Potanin", email = "bogdanpotanin@gmail.com", role = c("aut", "cre", "ctb")), person("Sofiia", "Dolgikh", email = "sidolgikh@gmail.com", role = c("ctb")))
Description: Implements genetic algorithm and particle swarm algorithm for real-valued functions. Various modifications (including hybridization and elitism) of these algorithms are provided. Implemented functions are based on ideas described in S. Katoch, S. Chauhan, V. Kumar (2020) <doi:10.1007/s11042-020-10139-6> and M. Clerc (2012) <https://hal.science/hal-00764996>.
Imports: Rcpp (>= 1.0.6)
LinkingTo: Rcpp, RcppArmadillo
License: GPL (>= 2)
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2026-04-08 11:13:31 UTC; Bogdan
Author: Bogdan Potanin [aut, cre, ctb],
  Sofiia Dolgikh [ctb]
Maintainer: Bogdan Potanin <bogdanpotanin@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-09 05:10:39 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-04-09 07:19:35 UTC; unix
Archs: gena.so.dSYM

Package: gena
Type: Package
Title: Genetic Algorithm and Particle Swarm Optimization
Version: 1.0.1
Date: 2026-04-08
Authors@R: c(person("Bogdan", "Potanin", email = "bogdanpotanin@gmail.com", role = c("aut", "cre", "ctb")), person("Sofiia", "Dolgikh", email = "sidolgikh@gmail.com", role = c("ctb")))
Description: Implements genetic algorithm and particle swarm algorithm for real-valued functions. Various modifications (including hybridization and elitism) of these algorithms are provided. Implemented functions are based on ideas described in S. Katoch, S. Chauhan, V. Kumar (2020) <doi:10.1007/s11042-020-10139-6> and M. Clerc (2012) <https://hal.science/hal-00764996>.
Imports: Rcpp (>= 1.0.6)
LinkingTo: Rcpp, RcppArmadillo
License: GPL (>= 2)
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2026-04-08 11:13:31 UTC; Bogdan
Author: Bogdan Potanin [aut, cre, ctb],
  Sofiia Dolgikh [ctb]
Maintainer: Bogdan Potanin <bogdanpotanin@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-09 05:10:39 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-09 08:20:17 UTC; unix
Archs: gena.so.dSYM

useDynLib(gena, .registration=TRUE)

importFrom(Rcpp, evalCpp)

importFrom("stats", "median", "optim", "pnorm", "qnorm", "rbinom",
           "rnorm", "runif")

importFrom("utils", "modifyList", "hasName")

export("gena", "gena.population", "gena.mating", 
       "gena.crossover", "gena.mutation", "gena.hybrid",
       "gena.constr")

export("gena.grad", "gena.hessian")

export("pso", "pso.velocity", "pso.nh")

export("rhypersphere")

S3method("print", "gena")
S3method("summary", "gena")
S3method("print", "summary.gena")
S3method("plot", "gena")

S3method("print", "pso")
S3method("summary", "pso")
S3method("print", "summary.pso")
S3method("plot", "pso")

# GENA

Framework to help to build (web) application faster.

1. [`gena`](): generate restful APIs from peewee models (i.e., specification of database table).
2. [`gena-app`](): provides basic structure for frontend application (state management, routing).

For demo, see the [todolist folder](/examples/todolist).

# Getting started

We are going to build a simple todolist app to demonstrate how to use this framework. See the [todolist folder](/todolist) for the complete source code.

### Setup the project:

1. create the python project (server code): `poetry new todolist` (you need [Poetry](https://python-poetry.org/) to run this command)
2. move inside the project folder: `cd todolist`
3. add our backend library: `poetry add gena`
4. create a `www` directory containing your frontend code using [`create-react-app`](https://create-react-app.dev/docs/adding-typescript/): (`yarn create react-app www --template typescript`)
5. add our frontend library: `cd www; yarn add gena-app; cd ..` -- this will modify your [`www/package.json`](/todolist/www/package.json) file.
6. modify the build script in [`www/package.json`](/todolist/www/package.json) to tell `create-react-app` to build static files into [`todolist/www`](/todolist/todolist/www) directory inside the python package: `"build": "BUILD_PATH='../todolist/www' react-scripts build"`. This allows us to distribute both frontend and backend in a single python package.
7. add `"proxy": "http://localhost:5000"` to the [`www/package.json`](/todolist/www/package.json). This allows us to send the request to the server during development.

After this step, you will have the following folder structure:

    todolist
    ├── todolist                   # our backend code
    │   ├── __init__.py
    ├── pyproject.toml
    ├── www                        # our frontend code

### Backend

#### Define database schema

We use [`peewee`](https://docs.peewee-orm.com/), an ORM library, to define our database schema.

First, let's create a file containing our schema: [`todolist/models.py`](/todolist/todolist/models.py). In this simple todo list, we only have one table `todo_list`, which has two fields: `checked` and `todo`.

```python
import os
from peewee import SqliteDatabase, Model, BooleanField, TextField


db = SqliteDatabase(os.environ["DBFILE"])


class TodoList(Model):
    class Meta:
        database = db
        db_table = "todo_list"

    checked = BooleanField()
    todo = TextField()
```

(Optionally) We add the following code to the same file to insert some dummy data to our database for testing

```python
if not os.path.exists(os.environ['DBFILE']):
    db.create_tables([TodoList], safe=True)
    TodoList.insert_many([
        {"checked": False, "todo": "go grocery"},
        {"checked": False, "todo": "do laundry"},
    ]).execute()
```

#### Create APIs

With our backend library, creating APIs is as simple as calling a function `generate_api` with the ORM model `TodoList`:

```python
import os
from gena import generate_app, generate_api
from todolist.models import TodoList

app = generate_app(
    controllers=[generate_api(model) for model in [TodoList]],
    pkg_dir=os.path.dirname(__file__)
)
```

Under the hood, [`generate_api`](/gena/api_generator.py) uses the specification in the model to automatically generate a blueprint containing the following endpoints (sometimes are called views or controllers):

1. `GET /{table_name}`: querying records matched a query.
2. `GET /{table_name}/<id>`: get a record by id
3. `HEAD /{table_name}/<id>`: whether a record with the given id exists.
4. `POST /{table_name}/find_by_ids`: querying list of records by their ids
5. `POST /{table_name}`: create a record
6. `PUT /{table_name}/<id>`: update a record
7. `DELETE /{table_name}/<id>`: delete a record
8. `DELETE /{table_name}`: truncate the whole table, only available if the `enable_truncate_table` parameter is set to be `True`.

You can extend the blueprint to add additional endpoints or overwrite some if you needed. Inputs from the users are automatically validate based on the type of the field, or you can provide your own deserializer (whose job is also validate the value) for a particular field via the `deserializer` parameter. You can also provide your own `serializer` to control how the data should be serialize.

The outer function `generate_app` will returns the Flask application. It takes three parameters

- `controllers`: a list of [Flask's blueprints](https://flask.palletsprojects.com/en/2.0.x/blueprints/) or a python package, in which the blueprints are discovered automatically. This is useful if you organize your blueprints in separated files. All endpoints defined in the blueprints have url prefix `/api` (`/{table_name}` becomes `/api/{table_name}`).
- `pkg_dir`: the path to our `todolist` package. Our app will serve the static files from `{pkg_dir}/www` folder.
- `log_sql_queries`: whether to log sql queries when we are in development mode.

With the given app, we finally can start our server with: `DBFILE=./todolist.db FLASK_APP=todolist/app.py flask run`. Note: remember to activate your virtual environment first with `poetry shell`.

### Frontend

#### Create Stores

Our frontend library uses [React](https://reactjs.org/) and provides state management with [MobX](https://mobx.js.org/)) and routing with [ReactRouter](https://reactrouter.com/) out of the box. In addition, most of the functions are typed so that refactoring and maintaining is easier.

To start, we create stores which contains all of the data of our application. A store also contains other functions to create, update, delete, and query records from the server. If you haven't familiar with MobX, please check out their [documentation](https://mobx.js.org/the-gist-of-mobx.html).

Let's create a file [`www/src/models/TodoListStore.ts`](/todolist/www/src/models/TodoListStore.ts) and paste the following code:

```typescript
import { SimpleCRUDStore } from "gena-app";
import { makeObservable, action } from "mobx";

export interface Todo {
  id: number;
  checked: boolean;
  todo: string;
}

export class TodoListStore extends SimpleCRUDStore<number, Todo> {
  constructor() {
    super(`/api/todo_list`);

    makeObservable(this, {
      toggle: action,
    });
  }

  toggle(item: Todo) {
    item.checked = !item.checked;
  }
}
```

In the above code, we create an interface `Todo` matched with the definition in our database schema. Since we didn't provide custom deserializer, the fields in our frontend is matched exactly with the fields in our database in the backend. Then, we define a store `TodoListStore` extending the `SimpleCRUDStore` from our frontend library `gena-app`. This gives us several useful functions out of the box such as `create`, `delete`, `update`, and `query` the records from the server. We also define an additional function that will toggle the `checked` value of an `Todo` to show how to extend the store if needed.

To use the stores with [React Hooks](https://reactjs.org/docs/hooks-intro.html), we need to following code in [`www/src/models/index.ts`](/todolist/www/src/models/index.ts)

```typescript
import React from "react";
import { TodoListStore } from "./TodoListStore";

export const stores = {
  todolistStore: new TodoListStore(),
};
export type IStore = Readonly<typeof stores>;
export const StoreContext = React.createContext<IStore>(stores);

export function useStores(): IStore {
  return React.useContext(StoreContext);
}

export { TodoListStore as VariableStore };
export type { Todo } from "./TodoListStore";
```

#### Routing & Application

Routing specifies which component the application should render for a particular URL. This makes it much easier to keep the application logic separated especially when you have lots of pages. Our frontend library enables us to define all the routing in a single place with strong type annotation, so that it's easier to maintain the application's routing and is harder to make mistake.

Let's create a file `www/routes.tsx` and define our first route:

```typescript
import { NoArgsPathDef } from "gena-app";
import { HomePage } from "./pages/HomePage";

export const routes = {
  home: new NoArgsPathDef({
    component: HomePage,
    pathDef: "/",
    exact: true,
  }),
};
```

In this route, we say when user open `/`, we will render a component [`HomePage`](/todolist/www/src/pages/HomePage.tsx). For now, let's just use a dummy `HomePage` for now:

```typescript
export const HomePage = () => {
  return <p>Hello world</p>;
};
```

With the specified routes and a dummy `HomePage` component, the final piece is to render and attach our component to the DOM. In [`index.tsx`](/todolist/www/src/index.tsx):

```typescript
import ReactDOM from "react-dom";
import { App } from "gena-app";
import "./index.css";
import { StoreContext, stores } from "./models";
import reportWebVitals from "./reportWebVitals";
import { routes } from "./routes";

ReactDOM.render(
  <StoreContext.Provider value={stores}>
    <App routes={routes} />
  </StoreContext.Provider>,
  document.getElementById("root")
);
```

`StoreContext.Provider` component enables us to use `useStores` hook we defined earlier in any component, and `App` component will render the component that matched with the current URL as specified in our routes.

#### Create TodoList component

It's now time to complete our [`HomePage`](/todolist/www/src/pages/HomePage.tsx) component to show the TodoList.

We first use the stores within the component and wrapped it with `observer` so that it will react whenever the state changes

```typescript
import { observer } from "mobx-react";

export const HomePage = observer(() => {
  const { todolistStore } = useStores();
  ...
});
```

We are going to display the todo list as a list. So we use [List component](https://ant.design/components/list).

```typescript
export const HomePage = observer(() => {
  const { todolistStore } = useStores();
  useEffect(() => {
    todolistStore.fetch({ limit: 1000, offset: 0 });
  }, []);

  const items = todolistStore.list.map((item) => {
    return (
      <List.Item key={item.id}>
        <Checkbox
          checked={item.checked}
          onChange={(e) => {
            item.checked = e.target.checked;
            todolistStore.update(item);
          }}
        >
          <Typography.Paragraph
            style={{ marginBottom: 0 }}
            editable={{
              onChange: (text) => {
                item.todo = text;
                todolistStore.update(item);
              },
            }}
          >
            {item.todo}
          </Typography.Paragraph>
        </Checkbox>
        <Button
          type="primary"
          danger={true}
          onClick={() => {
            todolistStore.delete(item.id);
          }}
        >
          Delete
        </Button>
      </List.Item>
    );
  });

  const addItem = () => todolistStore.create({ checked: false, todo: "" });

  return (
    <Space direction="vertical" style={{ width: "100%" }}>
      <List bordered={true}>{items}</List>
      <Button type="primary" onClick={addItem}>
        Add
      </Button>
    </Space>
  );
});
```

Now, we are able to display the list of todo items from the database, and check/uncheck item that is done. We can extend the UI to add other functionalities such as add, delete, or update the content of todo items. You can check out the complete code in [here](/todolist/www/src/pages/HomePage.tsx).

#' Genetic Algorithm
#' @description This function allows the use of a genetic algorithm for
#' numeric global optimization of real-valued functions.
#' @param fn function to be maximized i.e. fitness function.
#' @param gr gradient of the \code{fn}.
#' @param lower lower bound of the search space.
#' @param upper upper bound of the search space.
#' @param pop.initial numeric matrix whose rows are chromosomes to be
#' included in the initial population. A numeric vector will be coerced to
#' a single-row matrix.
#' @param pop.n integer representing the size of the population.
#' @param pop.method the algorithm to be applied for the creation of 
#' the initial population. See 'Details' for additional information.
#' @param mating.method the algorithm to be applied for mating, i.e., selection
#' of parents. See 'Details' for additional information.
#' @param mating.par parameters of the mating (selection) algorithm.
#' @param mating.self logical; if \code{TRUE} then the chromosome may
#' mate with itself, i.e., both parents may be the same chromosome.
#' @param crossover.method an algorithm to be applied for crossover, i.e.,
#' creation of the children. See 'Details' for additional information.
#' @param crossover.par parameters of the crossover algorithm.
#' @param crossover.prob probability of the crossover for each pair of parents.
#' @param mutation.method algorithm to be applied for mutation, i.e., random
#' change in some genes of the children. 
#' See 'Details' for additional information.
#' @param mutation.par parameters of the mutation algorithm.
#' @param mutation.prob mutation probability for the chromosomes.
#' @param mutation.genes.prob mutation probability for the genes.
#' @param elite.n number of elite children, i.e., those that have the
#' highest function value and will be preserved for the next population.
#' @param elite.duplicates logical; if \code{TRUE} then some elite children
#' may have the same genes.
#' @param hybrid.method hybrid selection algorithm, i.e., mechanism 
#' determining which chromosomes should be subject to local optimization.
#' See 'Details' for additional information.
#' @param hybrid.par parameters of the hybridization algorithm.
#' @param hybrid.prob probability of generating hybrids each iteration.
#' @param hybrid.opt.par parameters of the local optimization function
#' to be used for the hybridization algorithm 
#' (including \code{fn} and \code{gr}).
#' @param hybrid.n number of hybrids that appear if hybridization
#' takes place during the iteration.
#' @param constr.method the algorithm to be applied for imposing constraints
#' on the chromosomes. See 'Details' for additional information.
#' @param constr.par parameters of the constraint algorithm.
#' @param maxiter maximum number of iterations of the algorithm.
#' @param is.max logical; if \code{TRUE} (default) then fitness function
#' will be maximized. Otherwise it will be minimized.
#' @param info logical; if \code{TRUE} (default) then some optimization-related 
#' information will be printed each iteration.
#' @param ... additional parameters to be passed to 
#' \code{fn} and \code{gr} functions.
#' @details To find information on particular methods available via the 
#' \code{pop.method},
#' \code{mating.method}, \code{crossover.method}, \code{mutation.method},
#' \code{hybrid.method}, and \code{constr.method}
#' arguments, please see the 'Details' section of 
#' \code{\link[gena]{gena.population}}, \code{\link[gena]{gena.crossover}},
#' \code{\link[gena]{gena.mutation}}, \code{\link[gena]{gena.hybrid}}, 
#' and \code{\link[gena]{gena.constr}}, respectively. For example to
#' find information on possible values of \code{mutation.method} and
#' \code{mutation.par} arguments see description of \code{method} and
#' \code{par} arguments of \code{\link[gena]{gena.mutation}} function.
#' 
#' It is possible to provide manually implemented functions for 
#' population initialization, mating, crossover, mutation, and hybridization. 
#' For example, a manual mutation function may be provided through
#' the \code{mutation.method} argument. It should have the same signature 
#' (arguments) as the \code{\link[gena]{gena.mutation}} function and return 
#' the same object, i.e., the matrix of chromosomes of the appropriate size.
#' Manually implemented functions for other operators (crossover, mating,
#' and so on) may be provided in a similar way.
#' 
#' By default, the function does not impose any constraints on the parameters.
#' If \code{constr.method = "bounds"}, then \code{lower} and \code{upper} 
#' constraints will be imposed. Lower bounds should be strictly smaller
#' than upper bounds.
#' 
#' Currently the only available termination condition is \code{maxiter}. We
#' are going to provide some additional termination conditions during
#' future updates.
#' 
#' Infinite values in \code{lower} and \code{upper} are substituted with
#' \code{-(.Machine$double.xmax * 0.9)} and \code{.Machine$double.xmax * 0.9},
#' respectively.
#' 
#' By default, if \code{gr} is provided, then the BFGS algorithm will be used 
#' inside \code{\link[stats]{optim}} during hybridization.
#' Otherwise, \code{Nelder-Mead} will be used. 
#' Manual values for \code{\link[stats]{optim}} arguments may be provided 
#' (as a list) through the \code{hybrid.opt.par} argument.
#' 
#' Arguments \code{pop.n} and \code{elite.n} should be even integers, and
#' \code{elite.n} should be greater than 2. If these arguments are odd integers, 
#' then they will be coerced to even integers by adding 1. 
#' Also, \code{pop.n} should be greater than \code{elite.n} by at least 2.
#' 
#' For more information on the genetic algorithm,
#' please see Katoch et al. (2020).
#' 
#' @return This function returns an object of class \code{gena} that is a list
#' containing the following elements:
#' \itemize{
#' \item \code{par} - chromosome (solution) with the highest fitness
#' (objective function) value.
#' \item \code{value} - value of \code{fn} at \code{par}.
#' \item \code{population} - matrix of chromosomes (solutions) from the 
#' last iteration of the algorithm.
#' \item \code{counts} - a two-element integer vector giving the number of
#' calls to \code{fn} and \code{gr} respectively.
#' \item \code{is.max} - identical to \code{is.max} input argument.
#' \item \code{fitness.history} - vector whose i-th element is the fitness
#' of the best chromosome in the i-th iteration.
#' \item \code{iter} - last iteration number.
#' }
#' 
#' @references S. Katoch, S. Chauhan, V. Kumar (2020). 
#' A review on genetic algorithm: past, present, and future.
#' \emph{Multimedia Tools and Applications}, 80, 8091-8126.
#' <doi:10.1007/s11042-020-10139-6>
#' 
#' @examples
#' ## Consider Ackley function
#' \donttest{
#' fn <- function(par, a = 20, b = 0.2)
#' {
#'   val <- a * exp(-b * sqrt(0.5 * (par[1] ^ 2 + par[2] ^ 2))) +
#'          exp(0.5 * (cos(2 * pi * par[1]) + cos(2 * pi * par[2]))) -
#'          exp(1) - a
#'   return(val)
#' }
#' 
#' # Maximize this function using classical
#' # genetic algorithm setup
#' set.seed(123)
#' lower <- c(-5, -100)
#' upper <- c(100, 5)
#' opt <- gena(fn = fn, 
#'             lower = lower, upper = upper,
#'             hybrid.prob = 0,
#'             a = 20, b = 0.2)
#' print(opt$par)
#' 
#' # Replicate optimization using hybridization
#' opt <- gena(fn = fn, 
#'             lower = lower, upper = upper,
#'             hybrid.prob = 0.2,
#'             a = 20, b = 0.2)
#' print(opt$par)
#' }
#' 
#' ## Consider Rosenbrock function
#' fn <- function(par, a = 100)
#' {
#'   val <- -(a * (par[2] - par[1] ^ 2) ^ 2 + (1 - par[1]) ^ 2 +
#'            a * (par[3] - par[2] ^ 2) ^ 2 + (1 - par[2]) ^ 2)
#'   return(val)
#' }
#' 
#' # Apply genetic algorithm
#' lower <- rep(-10, 3)
#' upper <- rep(10, 3)
#' set.seed(123)
#' opt <- gena(fn = fn,
#'             lower = lower, upper = upper,
#'             a = 100)
#' print(opt$par)
#' 
#' \donttest{
#' # Improve the results by hybridization
#' opt <- gena(fn = fn,
#'             lower = lower, upper = upper,
#'             hybrid.prob = 0.2,
#'             a = 100)
#' print(opt$par)
#' }
#' 
#' # Provide manually implemented mutation function
#' # which simply randomly sorts genes. 
#' # Note that this function should have the same 
#' # arguments as gena.mutation.
#' mutation.my <- function(children, lower, upper, 
#'                         prob, prob.genes, 
#'                         method, par, iter)
#' {
#'   # Get dimensional data
#'   children.n <- nrow(children)
#'   genes.n <- ncol(children)
#'   
#'   # Select chromosomes that should mutate
#'   random_values <- runif(children.n, 0, 1)
#'   mutation_ind <- which(random_values <= prob)
#'    
#'   # Mutate chromosomes by randomly sorting
#'   # their genes
#'   for (i in mutation_ind)
#'   {
#'     children[i, ] <- children[i, sample(1:genes.n)]
#'   }
#'   
#'   # Return mutated chromosomes
#'   return(children)
#' }
#' 
#' opt <- gena(fn = fn,
#'             lower = lower, upper = upper,
#'             mutation.method = mutation.my,
#'             a = 100)
#' print(opt$par)
#' 
gena <- function(
  fn,
  gr = NULL,
  lower,
  upper,
  pop.n = 100,
  pop.initial = NULL,
  pop.method = "uniform",
  mating.method = "rank",
  mating.par = NULL,
  mating.self = FALSE,
  crossover.method = "local",
  crossover.par = NULL,
  crossover.prob = 0.8,
  mutation.method = "constant",
  mutation.par = NULL,
  mutation.prob = 0.2,
  mutation.genes.prob = 1 / length(lower),
  elite.n = min(10, 2 * round(pop.n / 20)),
  elite.duplicates = FALSE,
  hybrid.method = "rank",
  hybrid.par = 2,
  hybrid.prob = 0,
  hybrid.opt.par = NULL,
  hybrid.n = 1,
  constr.method = NULL,
  constr.par = NULL,
  maxiter = 100,
  is.max = TRUE,
  info = TRUE,
  ...)
{
  # Initialize some variables
  
  pop.n <- ceiling(pop.n)
  elite.n <- ceiling(elite.n)
  genes.n <- length(lower)
  
  # Perform initial validation
  
    # fn
  
  if (!is.function(fn))
  {
    stop("Please, ensure that 'fn' is a function.\n")
  }
  
    # gr
  if(!is.null(gr))
  {
    if (!is.function(gr))
    {
      stop("Please, ensure that 'gr' is a function.\n")
    }
  }
  
    # lower and upper
  
  if (length(lower) != length(upper))
  {
    stop("Please, ensure that 'lower' and 'upper' are of the same length.\n")
  }
  
  if (any(lower > upper))
  {
    stop(paste0("Please, ensure that all elements of the 'lower' are ",
                "smaller than the corresponding elements of the 'upper'.",
                "\n"))
  }
  
  lower[is.infinite(lower)] <- -(.Machine$double.xmax * 0.9)
  upper[is.infinite(upper)] <- .Machine$double.xmax * 0.9
  
    # pop.initial
  
  if (!is.null(pop.initial))
  {
    if (!is.numeric(pop.initial))
    {
      stop("Please, ensure that 'pop.initial' is a numeric matrix.\n")
    }
    
    if (!is.matrix(pop.initial))
    {
      pop.initial <- matrix(pop.initial, nrow = 1)
    }

    if (ncol(pop.initial) != genes.n)
    {
      stop(paste0("Please, ensure that the number of columns of 'pop.initial' ",
                  "equals the number of estimated parameters, i.e., genes.",
                  "\n"))
    }
  }
  
    # pop.n
  
  if (pop.n <= 1)
  {
    stop("Please, ensure that 'pop.n' is an integer greater than one.")
  }
  
  if ((pop.n %% 2) != 0) 
  {
    pop.n <- pop.n + 1
    warning("Since 'pop.n' is not even, it has been incremented by one.\n")
  }
  
    # elite.n
  
  if (elite.n <= 1)
  {
    stop("Please, ensure that 'elite.n' is an integer greater than one.\n")
  }
  
  if ((elite.n %% 2) != 0) 
  {
    elite.n <- elite.n + 1
    warning("Since 'elite.n' is not even, it has been incremented by one.\n")
  }
    
  # Initialize some additional variables  
  
  parents.n <- pop.n - elite.n
  fitness <- rep(NA, pop.n)
  
  # Continue validation
  
    # method.mating and mating.par
  if (!is.function(mating.method))
  {
    mating.par <- gena.mating.validate(method = mating.method,
                                       par = mating.par,
                                       parents.n = parents.n)
  }
  else
  {
    gena.mating <- mating.method
  }
  
    # mating.self
  
  if (!is.logical(mating.self))
  {
    if ((mating.self == 0) | (mating.self == 1))
    {
      mating.self <- as.logical(mating.self)
    } else {
      stop("Please, ensure that 'mating.self' is a logical.\n")
    }
  }
  
    # crossover.method and crossover.par
  
  if (!is.function(crossover.method))
  {
    crossover.par <- gena.crossover.validate(method = crossover.method,
                                             par = crossover.par)
  }
  else
  {
   gena.crossover <- crossover.method
  }
  
    # crossover.prob  
  
  if (is.numeric(crossover.prob))
  {
    if ((crossover.prob < 0) | (crossover.prob > 1))
    {
      stop("Please, ensure that 'crossover.prob' is between 0 and 1.\n")
    }
  } else {
    stop("Please, ensure that 'crossover.prob' is a numeric value.\n")
  }
  
    # mutation.method and mutation.par
  
  if (!is.function(mutation.method))
  {
    mutation.par <- gena.mutation.validate(method = mutation.method,
                                           par = mutation.par,
                                           genes.n = genes.n)
  }
  else
  {
    gena.mutation <- mutation.method
  }
  
    # mutation.prob
  
  if (is.numeric(mutation.prob))
  {
    if ((mutation.prob < 0) | (mutation.prob > 1))
    {
      stop("Please, ensure that 'mutation.prob' is between 0 and 1.\n")
    }
  } else {
    stop("Please, ensure that 'mutation.prob' is a numeric value.\n")
  }
  
    # mutation.genes.prob
  
  mutation.genes.prob_n <- length(mutation.genes.prob)
  
  if (mutation.genes.prob_n == 1)
  {
    mutation.genes.prob <- rep(mutation.genes.prob, genes.n)
  }
  
  if (mutation.genes.prob_n > genes.n)
  {
    stop(paste0("Please, ensure that the length of 'mutation.genes.prob' ",
                "equals 1 or the same as the length of 'lower'.",
                "\n"))
  }
  
  if (any(mutation.genes.prob < 0) | any(mutation.genes.prob > 1))
  {
    stop(paste0("Please, ensure that values in 'mutation.genes.prob' ",
                "are between 0 and 1.",
                "\n"))
  }
  
  if ((pop.n - elite.n) < 2)
  {
    stop(paste0("Please, ensure that ((pop.n - elite.n) >= 2), i.e., ",
                "it seems that 'elite.n' is too large relative to 'pop.n'.",
                "\n"))
  }
  
    # elite.duplicates
  
  if (!is.logical(elite.duplicates))
  {
    if ((elite.duplicates == 0) | (elite.duplicates == 1))
    {
      elite.duplicates <- as.logical(elite.duplicates)
    } else {
      stop("Please, ensure that 'elite.duplicates' is a logical.\n")
    }
  }
  
    # hybrid.method and hybrid.par
  
  if (!is.function(hybrid.method))
  {
    hybrid.par <- gena.mating.validate(method = hybrid.method,
                                       par = hybrid.par,
                                       parents.n = parents.n)
  }
  else
  {
    gena.hybrid <- hybrid.method
  }
  
    # hybrid.prob
  
  if (is.numeric(hybrid.prob))
  {
    if ((hybrid.prob < 0) | (hybrid.prob > 1))
    {
      stop("Please, ensure that 'hybrid.prob' is between 0 and 1.\n")
    }
  } else {
    stop("Please, ensure that 'hybrid.prob' is a numeric value.\n")
  }
  
    # hybrid.n
  
  if (hybrid.n < 0)
  {
    stop("Please, ensure that 'hybrid.n' is a non-negative integer.\n")
  }
  
  if (hybrid.n > pop.n)
  {
    stop("Please, ensure that 'hybrid.n' is not greater than 'pop.n'.")
  }
  
    # maxiter
  
  if (maxiter < 1)
  {
    stop("Please, ensure that 'maxiter' is a positive integer.")
  }
  
    # constr.method and constr.par
  
  if(!is.null(constr.method))
  {
    if (!(is.function(constr.method) | is.character(constr.method)))
    {
      stop("Please, ensure that 'constr.method' is a function or character.\n")
    }
    if (is.character(constr.method))
    {
      if ((constr.method == "bounds") & is.null(constr.par))
      {
        constr.par$lower <- lower
        constr.par$upper <- upper
      }
      gena.constr.validate(method = constr.method, par = constr.par)
    }
    if (is.function(constr.method))
    {
      gena.constr <- constr.method
    }
  }
  
    # is.max
    
  if (!is.logical(is.max))
  {
    if ((is.max == 0) | (is.max == 1))
    {
      is.max <- as.logical(is.max)
    } else {
      stop("Please, ensure that 'is.max' is a logical.\n")
    }
  }
  
    # info
  
  if (!is.logical(info))
  {
    if ((info == 0) | (info == 1))
    {
      info <- as.logical(info)
    } else {
      stop("Please, ensure that 'info' is a logical.\n")
    }
  }
  
  # Value to reverse signs of fitnesses if needed
  
  is.max.val <- 2 * is.max - 1
  
  # Deal with hybrid optimizer default parameters
  
  hybrid.opt.par.default = list(fn = fn,
                                gr = gr,
                                method = ifelse(is.null(gr), 
                                                "Nelder-Mead", 
                                                "BFGS"),
                                control = list(fnscale = -1 * is.max.val,
                                               abstol = 1e-10,
                                               reltol = 1e-10,
                                               maxit = 1000000000))
  
  if (!is.null(hybrid.opt.par))
  {
    if (!is.null(hybrid.opt.par$control))
    {
      hybrid.opt.par.default$control <- modifyList(hybrid.opt.par.default$control,
                                                   hybrid.opt.par$control)
    }
    hybrid.opt.par <- modifyList(hybrid.opt.par.default, 
                                 hybrid.opt.par)
  } else {
    hybrid.opt.par <- hybrid.opt.par.default
  }

  # Initialize the population
  
  if (is.function(pop.method))
  {
    gena.population <- pop.method
  }
  
  population <- gena.population(pop.n = pop.n,
                                lower = lower, 
                                upper = upper,
                                method = pop.method,
                                pop.initial = pop.initial)
  
  # Number of function and gradient evaluations
  
  counts <- c(0, 0)
  names(counts) <- c("function", "gradient")
  
  # Get the name of the first argument of the function
  
  fn.par <- names(formals(fn))[1]
  
  # Historical best fitness values
  
  fitness.history <- NULL
  
  # -------------------------------------
  # Perform the genetic algorithm routine
  # -------------------------------------
  
  for(i in 1:maxiter)
  {
    # Get the parameters to be passed to fitness
    
    dots <- list(...)
    
    # Impose the constraints on the parameters
    
    if (!is.null(constr.method))
    {
      if (is.function(constr.method))
      {
        constr.par$population <- population
        constr.par$iter <- i
        population <- do.call(what = constr.method, 
                              args = constr.par)$population
      }
      else
      {
        population <- gena.constr(population = population,
                                  iter = i,
                                  method = constr.method,
                                  par = constr.par)$population
      }
    }

    # Estimate the fitness of each chromosome

    for(j in 1:pop.n)
    {
      dots[[fn.par]] <- population[j, ] 
      fitness[j] <- do.call(what = fn, args = dots)
    }
    fitness[is.na(fitness)] <- -Inf
    dots[[fn.par]] <- NULL
    
    # Make the hybrids
    
    hybrid.value <- runif(1, 0, 1)
    
    if (((hybrid.value <= hybrid.prob) |
         (i == 1) | (i == maxiter)) & 
        (hybrid.n >= 1) & (hybrid.prob > 0))
    {
      hybrids.list <- gena.hybrid(population = population,
                                  fitness = fitness,
                                  hybrid.n = hybrid.n,
                                  method = hybrid.method,
                                  par = hybrid.par,
                                  opt.par = hybrid.opt.par,
                                  info = info,
                                  ... = ...)
      population <- hybrids.list$population
      fitness <- hybrids.list$fitness
      counts <- hybrids.list$counts + counts
    }

    # Reverse signs of fitnesses for minimization task
    
    if (!is.max)
    {
      fitness <- -fitness
    }
    
    # Get best chromosome and store its fitness value
    
    best.ind <- which.max(fitness)
    fitness.history <- c(fitness.history, fitness[which.max(fitness)])
    
    # Return the results if needed
    
    if (i == maxiter)
    {
      counts[1] <- counts[1] + i * pop.n
      return.list <- list(population = population,
                          par = population[best.ind, ],
                          value = is.max.val * fitness[best.ind],
                          counts = counts,
                          is.max = is.max,
                          fitness.history = is.max.val * fitness.history,
                          iter = i)
      class(return.list) <- "gena"
      return(return.list)
    }
    
    # Initialize the parents
    
    parents.list <- gena.mating(population = population,
                                parents.n = parents.n,
                                fitness = fitness,
                                method = mating.method,
                                self = mating.self,
                                par = mating.par)
    parents <- parents.list$parents
    parents_fitness <- parents.list$fitness

    # Make the children via the crossover
    
    children.co <- NULL
    
    children.co <- gena.crossover(parents = parents,
                                  fitness = parents_fitness,
                                  prob = crossover.prob,
                                  method = crossover.method,
                                  par = crossover.par,
                                  iter = i)
    
    # Make the elite children (no need for parents, just the best chromosomes)
    
    fitness.order <- order(fitness, decreasing = TRUE)
    children.elite <- population[fitness.order[1:elite.n], , drop = FALSE]

    # Remove duplicate elite if needed
    
    if(!elite.duplicates)
    {
      elite.counter <- 2
      j.prev <- fitness.order[1]
      for (j in fitness.order[-1])
      {
        if (elite.counter > elite.n)
        {
          break
        }

        if (all(population[j, , drop = FALSE] != population[j.prev, , drop = FALSE]))
        {
          children.elite[elite.counter, ] <- population[j, , drop = FALSE]
          elite.counter <- elite.counter + 1
        }

        j.prev <- j
      }
    }

    # Mutate the children

    children.co <- gena.mutation(children = children.co,
                                 lower = lower, 
                                 upper = upper,
                                 method = mutation.method,
                                 prob = mutation.prob,
                                 prob.genes = mutation.genes.prob,
                                 par = mutation.par,
                                 iter = i)
    
    # Combine crossover and elite children to
    # make a new population

    population <- rbind(children.elite, children.co)

    if (info)
    {
      cat(paste0("Iter   = ", i, ", ",
                 "mean   = ", is.max.val * mean(fitness),   ", ",
                 "median = ", is.max.val * median(fitness), ", ",
                 "best   = ", is.max.val * max(fitness),    "\n"))
    }
  }

  return(NULL)
}

#' Print method for "gena" object
#' @param x Object of class "gena"
#' @param ... further arguments (currently ignored)
#' @returns This function does not return anything.
print.gena <- function(x, ...)
{
  cat("We have optimized, optimized, and finally optimized! \n")
  cat("---\n")
  cat(paste0(ifelse(x$is.max, "The largest ", "The smallest "),
             "value of the fitness function found is ", 
             signif(x$value, 5), " at point: \n"))
  cat(signif(x$par, 5))
  cat("\n")
  cat("---\n")
  cat(paste0("There were ", x$counts[1], " calls to the function and ",
             x$counts[2], " calls to the gradient. \n"))
  cat("---\n")
  cat("This object is a list containing the following elements:\n")
  cat(names(x))
}

#' Summarizing gena Fits
#' @param object Object of class "gena"
#' @param ... further arguments (currently ignored)
#' @return This function returns the same list as \code{\link[gena]{gena}} 
#' function changing its class to "summary.gena".
summary.gena <- function(object, ...)
{
  if (length(list(...)) > 0)
  {
    warning("Additional arguments passed through ... are ignored.")   
  }
  class(object) <- "summary.gena"
  return(object)
}

#' Summary for "gena" object
#' @param x Object of class "gena"
#' @param ... further arguments (currently ignored)
#' @returns This function returns \code{x} input argument.
print.summary.gena <- function (x, ...) 
{
  if (length(list(...)) > 0)
  {
    warning("Additional arguments passed through ... are ignored.")   
  }
  print.gena(x)
  return(x)
}

#' Plot best found fitness during genetic algorithm
#' @param x Object of class "gena"
#' @param y this parameter currently ignored
#' @param ... further arguments (currently ignored)
#' @returns This function does not return anything.
plot.gena <- function (x, y = NULL, ...) 
{
  if (length(list(...)) > 0)
  {
    warning("Additional arguments passed through ... are ignored.")   
  }
  plot(x = 1:x$iter, y = x$fitness.history, 
       main = "Historical values of fitness", 
       xlab = "Iteration", 
       ylab = paste0(ifelse(x$is.max, "The largest ", "The smallest "),
                     "fitness value"))
}

#' Neighbourhood
#' @description Constructs a neighbourhood of each particle using 
#' particular topology.
#' @param pop.n integer representing the size of the population.
#' @param method string representing the topology to be used for construction
#' of the neighbourhood. See 'Details' for additional information.
#' @param par additional parameters to be passed depending on the \code{method}.
#' @param iter iteration number of the genetic algorithm.
#' @details If \code{method = "ring"} then each particle will have 
#' \code{par[1]} neighbours. By default \code{par[1] = 3}.
#' See section 3.2.1 of M. Clerc (2012) for 
#' additional details.
#' If \code{method = "wheel"} then there is a single (randomly selected) 
#' particle which informs (and is informed by) other particles, while there is 
#' no direct communication among other particles. 
#' If \code{method = "random"} then each particle randomly informs other
#' \code{par[1]} particles and itself. Note that duplicates are possible, so 
#' sometimes each particle may inform fewer than \code{par[1]} particles. 
#' By default \code{par[1] = 3}.
#' See section 3.2.2 of M. Clerc (2012) for more details.
#' If \code{method = "star"} then all particles are fully informed
#' by each other.
#' If \code{method = "random2"} then each particle will be self-informed
#' and is informed by the j-th particle with probability \code{par[1]}
#' (value between 0 and 1). By default \code{par[1] = 0.1}.
#' @return This function returns a list whose i-th element is a vector of
#' particle indices that inform the i-th particle, i.e., neighborhood of the
#' i-th particle.
#' 
#' @references Maurice Clerc (2012). 
#' Standard Particle Swarm Optimisation.
#' \emph{HAL archive}.
#' 
#' @examples 
#' # Prepare random number generator
#' set.seed(123)
#' 
#' # Ring topology with 5 neighbours
#' pso.nh(pop.n = 10, method = "ring", par = 5)
#' 
#' # Wheel topology
#' pso.nh(pop.n = 10, method = "wheel")
#' 
#' # Star topology
#' pso.nh(pop.n = 10, method = "star")
#' 
#' # Random topology where each particle 
#' # randomly informs 3 other particles
#' pso.nh(pop.n = 10, method = "random", par = 3)
#' 
#' # Random2 topology where each particle could
#' # be informed by another with probability 0.2
#' pso.nh(pop.n = 10, method = "random2", par = 0.2)
#' 
pso.nh <- function(pop.n = 40,
                   method = "ring",
                   par = 3,
                   iter = 1)              
{
  # Prepare list to store neighbourhood
  # of each particle
  
  nh <- vector(mode = "list", length = pop.n)
  
  if (method == "ring")
  {
    nh.adj <- floor((par[1] - 1) / 2)
    is.odd <- (par[1] %% 2 == 0)
    nh.elements <- c((pop.n - nh.adj + 1):pop.n,
                     1:pop.n,
                     1:(nh.adj + is.odd))
    nh.ind <- 0:(par[1] - 1)
    for (i in 1:pop.n)
    {
      nh[[i]] <- nh.elements[nh.ind + i]
    }
  }
  
  if (method == "wheel")
  {
    wheel.particle = sample(1:pop.n, size = 1)
    nh.mat <- cbind(wheel.particle, 1:pop.n)
    nh <- as.list(as.data.frame(t(nh.mat)))
  }
  
  if (method == "star")
  {
    for (i in 1:pop.n)
    {
      nh[[i]] <- 1:pop.n
    }
  }
  
  if (method == "random")
  {
    pop.ind <- 1:pop.n
    for (i in pop.ind)
    {
      nh[[i]] <- i
    }
    for (i in pop.ind)
    {
      informs <- sample(x = pop.ind[-i], 
                        size = par[1], 
                        replace = TRUE)
      for (j in informs)
      {
        nh[[j]] <- c(nh[[j]], i)
      }
    }
    for (i in pop.ind)
    {
      nh[[i]] <- unique(nh[[i]])
    }
  }
  
  if (method == "random2")
  {
    pop.ind <- 1:pop.n
    for (i in pop.ind)
    {
      informs.ind <- as.logical(rbinom(n = pop.n - 1, size = 1, prob = par[1]))
      nh[[i]] <- c(i, pop.ind[-i][informs.ind])
    }
  }
  
  return(nh)
}

# Assign default parameters for
# neighbourhood algorithm depending
# on the "method"
pso.nh.validate <- function(method, par, pop.n)
{
  # Validate the "method"
  
  methods <- c("ring", "random", "random2",                # the list of all
               "star", "wheel")                            # available methods
                                                           
  if (!(method %in% methods))
  {
    stop(paste0("Incorrect 'nh.method' argument. ",        # if the user has provided
                "It should be one of: ",                   # incorrect argument
                paste(methods, collapse = ", "),
                "\n"))
  }
  
  if (method %in% c("ring"))
  {
    if (!is.null(par))
    {
      if ((par[1] > pop.n) | (par[1] < 3))
      {
        stop(paste0("Incorrect 'nh.par' argument. Please, ensure that ",
                    "'nh.par' is not less than 3 and not greater",
                    "than 'pop.n'. \n"))
      }
    }
  }
  
  if (method %in% c("random"))
  {
    if (!is.null(par))
    {
      if (!is.numeric(par))
      {
        stop(paste0("Incorrect 'nh.par' argument. Please, ensure that ",
                    "'nh.par' is numeric. \n"))
      }
    } else {
      par <- 3
    }
  }
  
  if (method %in% c("random2"))
  {
    if (!is.null(par))
    {
      if ((par[1] < 0) | (par[1] > 1))
      {
        stop(paste0("Incorrect 'nh.par' argument. Please, ensure that ",
                    "'nh.par' is numeric value between 0 and 1. \n"))
      }
    }
    else
    {
      par <- 0.1
    }
  }
  
  return(par)
}

#' Constraints
#' @description Impose constraints on chromosomes.
#' @param population numeric matrix which rows are chromosomes i.e. vectors of 
#' parameters values.
#' @param method method used to impose constraints.
#' @param par additional parameters to be passed depending on the \code{method}.
#' @param iter iteration number of the genetic algorithm.
#' @details If \code{method = "bounds"} then chromosomes will be bounded
#' between \code{par$lower} and \code{par$upper}.
#' @return The function returns a list with the following elements:
#' \itemize{
#' \item \code{population} - matrix which rows are chromosomes after
#' constraints have been imposed.
#' \item \code{constr.ind} - matrix of logical values which (i, j)-th
#' elements equals \code{TRUE} (\code{FALSE} otherwise) if j-th jene of
#' i-th chromosome is a subject to constraint.
#' }
#' @examples 
#' # Randomly initialize population
#' set.seed(123)
#' population <- gena.population(pop.n = 10,
#'                               lower = c(-5, -5), 
#'                               upper = c(5, 5))
#'                            
#' # Impose lower and upper bounds constraints
#' pop.constr <- gena.constr(population, 
#'                           method = "bounds",
#'                           par = list(lower = c(-1, 2),
#'                                      upper = c(1, 5)))
#' print(pop.constr)
#' 
gena.constr <- function(population,
                        method = "bounds",
                        par,
                        iter)              
{
  # Get the number of genes
  
  genes.n <- ncol(population)
  pop.n <- nrow(population)
  
  # Transform lower and upper bounds into matrices
  
  lower.mat <- matrix(par$lower, 
                      nrow = pop.n, ncol = genes.n, byrow=TRUE)
  upper.mat <- matrix(par$upper, 
                      nrow = pop.n, ncol = genes.n, byrow=TRUE)
  
  # Prepare the output list
  
  return.list <- list(population = NA,
                      constr.ind = NA)

  if (method == "bounds")
  {
      constr.ind.lower <- population < lower.mat
      constr.ind.upper <- population > upper.mat
      constr.ind <- constr.ind.lower | constr.ind.upper

      population[constr.ind.lower] <- lower.mat[constr.ind.lower]
      population[constr.ind.upper] <- upper.mat[constr.ind.upper]
      
      return.list$population <- population
      return.list$constr.ind <- constr.ind
  }

  return(return.list)
}

# Assign default parameters for
# constraint algorithm depending
# on the "method"
gena.constr.validate <- function(method, par)
{
  # Validate the "method"
  
  methods <- c("bounds")                                   # the list of all
                                                           # available methods
  
  if (!(method %in% methods))
  {
    stop(paste0("Incorrect constr.method argument. ",      # if the user has provided
                "It should be one of: ",                   # incorrect argument
                paste(methods, collapse = ", "),
                "\n"))
  }
}

#' Hypersphere
#' @description Simulates uniform random variates from the hypersphere.
#' @param n number of observations to simulate.
#' @param dim dimension of the hypersphere.
#' @param radius radius of the hypersphere.
#' @param center center of  the hypersphere.
#' @param type character; if \code{"boundary"} (default) then random variates 
#' are simulated from the hypersphere. If \code{"inside"}, the random
#' variates are points lying inside the hypersphere. If \code{"non-uniform"}, 
#' then random variates are non-uniform and are simulated from the inner part
#' of the hypersphere by making the radius a uniform random variable
#' between \code{0} and \code{radius}.
#' @return The function returns a matrix whose rows are random variates 
#' (when \code{n > 1}) or a vector (when \code{n = 1}).
#' @examples 
#' set.seed(123)
#' # Get 5 random uniform variates from 3D hypersphere
#' # of radius 10 centered at (2, 3, 1)
#' rhypersphere(n = 5, dim = 3, radius = 10, center = c(2, 3, 1))
#' 
rhypersphere <- function(n, dim = 2,
                         radius = 1, center = rep(0, dim),
                         type = "boundary")              
{
  # Generate independent standard normal variates
  
  x <- matrix(rnorm(n * dim), nrow = n, ncol = dim)
  
  # Transform these variates
  
  y <- sweep(x, 1, sqrt(rowSums(x ^ 2)), "/")
  
  # Select points inside sphere or on the boundary
  
  if (type == "inside")
  {
    radius <- (radius * runif(n)) ^ (1 / dim)
  }
  
  if (type == "non-uniform")
  {
    radius <- radius * runif(n)
  }
  
  # Adjust for radius
  
  if (length(radius) == 1)
  {
    y <- y * radius
  }
  else
  {
    y <- sweep(y, 1, radius, "*")
  }

  # Adjust for center
  
  if (!is.matrix(center))
  {
    y <- sweep(y, 2, center, "+")
  }
  else
  {
    y <- y + center
  }
  
  # Return the results
  
  return(y)
}

#' Particle Swarm Optimization
#' @description This function allows the use of the particle swarm algorithm for
#' numeric global optimization of real-valued functions.
#' @param fn function to be maximized, i.e., fitness function.
#' @param gr gradient of the \code{fn}.
#' @param lower lower bound of the search space.
#' @param upper upper bound of the search space.
#' @param pop.initial numeric matrix whose rows are particles to be
#' included in the initial population. A numeric vector will be coerced to
#' a single-row matrix.
#' @param pop.n integer representing the size of the population.
#' @param pop.method the algorithm to be applied for a creation of 
#' the initial population. See 'Details' for additional information.
#' @param nh.method string representing the method (topology) to be used for
#' the creation of neighbourhoods. See 'Details' for additional information.
#' @param nh.par parameters of the topology algorithm.
#' @param nh.adaptive logical; if \code{TRUE} (default), then neighbourhoods
#' change every time the best known (to the swarm) fitness value has 
#' not increased. Neighbourhoods are updated according to the topology
#' defined via \code{nh.method} argument.
#' @param velocity.method string representing the method to be used for
#' the update of velocities.
#' @param velocity.par parameters of the velocity formula.
#' @param hybrid.method hybrid selection algorithm, i.e., mechanism 
#' determining which particles should be subject to local optimization.
#' See 'Details' for additional information.
#' @param hybrid.par parameters of the hybridization algorithm.
#' @param hybrid.prob probability of generating hybrids each iteration.
#' @param hybrid.opt.par parameters of the local optimization function
#' to be used for hybridization algorithm (including \code{fn} and \code{gr}).
#' @param hybrid.n number of hybrids that appear if hybridization
#' should take place during the iteration.
#' @param constr.method the algorithm to be applied for imposing constraints
#' on the particles. See 'Details' for additional information.
#' @param constr.par parameters of the constraint algorithm.
#' @param random.order logical; if \code{TRUE} (default), then 
#' particles-related routine will be implemented in a random order.
#' @param maxiter maximum number of iterations of the algorithm.
#' @param is.max logical; if \code{TRUE} (default) then fitness function
#' will be maximized. Otherwise it will be minimized.
#' @param info logical; if \code{TRUE} (default) then some optimization related 
#' information will be printed each iteration.
#' @param ... additional parameters to be passed to 
#' \code{fn} and \code{gr} functions.
#' 
#' @details Default arguments have been set in accordance with SPSO 2011
#' algorithm proposed by M. Clerc (2012).
#' 
#' To find information on particular methods available via
#' the \code{pop.method}, \code{nh.method}, \code{velocity.method},
#' \code{hybrid.method}, and \code{constr.method}
#' arguments, please see the 'Details' section of 
#' \code{\link[gena]{gena.population}}, \code{\link[gena]{pso.nh}},
#' \code{\link[gena]{pso.velocity}}, \code{\link[gena]{gena.hybrid}}, 
#' and \code{\link[gena]{gena.constr}}, respectively.
#' 
#' It is possible to provide manually implemented functions for population
#' initialization, neighbourhood creation, velocity update, hybridization,
#' and constraints in a similar way as for \code{\link[gena]{gena}}.
#' 
#' By default, the function does not impose any constraints on the parameters.
#' If \code{constr.method = "bounds"}, then \code{lower} and \code{upper} 
#' constraints will be imposed. Lower bounds should be strictly smaller
#' than upper bounds.
#' 
#' Currently the only available termination condition is \code{maxiter}. We
#' are going to provide some additional termination conditions during
#' future updates.
#' 
#' Infinite values in \code{lower} and \code{upper} are substituted with
#' \code{-(.Machine$double.xmax * 0.9)} and \code{.Machine$double.xmax * 0.9},
#' respectively.
#' 
#' By default if \code{gr} is provided, then the BFGS algorithm will be used 
#' inside \code{\link[stats]{optim}} during hybridization.
#' Otherwise, \code{Nelder-Mead} will be used. 
#' Manual values for \code{\link[stats]{optim}} arguments may be provided 
#' (as a list) through \code{hybrid.opt.par} the argument.
#' 
#' For more information on particle swarm optimization, 
#' please see M. Clerc (2012).
#' 
#' @return This function returns an object of class \code{pso} that is a list
#' containing the following elements:
#' \itemize{
#' \item \code{par} - particle (solution) with the highest fitness
#' (objective function) value.
#' \item \code{value} - value of \code{fn} at \code{par}.
#' \item \code{population} - matrix of particles (solutions) from the 
#' last iteration of the algorithm.
#' \item \code{counts} - a two-element integer vector giving the number of
#' calls to \code{fn} and \code{gr} respectively.
#' \item \code{is.max} - identical to \code{is.max} input argument.
#' \item \code{fitness.history} - vector whose i-th element is the fitness
#' of the best particle in the i-th iteration.
#' \item \code{iter} - last iteration number.
#' }
#' 
#' @references M. Clerc (2012). 
#' Standard Particle Swarm Optimisation.
#' \emph{HAL archive}.
#' 
#' @examples
#' ## Consider Ackley function
#' \donttest{
#' fn <- function(par, a = 20, b = 0.2)
#' {
#'   val <- a * exp(-b * sqrt(0.5 * (par[1] ^ 2 + par[2] ^ 2))) +
#'          exp(0.5 * (cos(2 * pi * par[1]) + cos(2 * pi * par[2]))) -
#'          exp(1) - a
#'   return(val)
#' }
#' 
#' # Maximize this function using particle swarm algorithm
#' 
#' set.seed(123)
#' lower <- c(-5, -100)
#' upper <- c(100, 5)
#' opt <- pso(fn = fn, 
#'            lower = lower, upper = upper,
#'            a = 20, b = 0.2)
#' print(opt$par)
#' }
#' 
#' ## Consider Bukin function number 6
#' 
#' fn <- function(x, a = 20, b = 0.2)
#' {
#'   val <- 100 * sqrt(abs(x[2] - 0.01 * x[1] ^ 2)) + 0.01 * abs(x[1] + 10)
#'   return(val)
#' }
#' 
#' # Minimize this function using initially provided
#' # position for one of the particles
#' set.seed(777)
#' lower <- c(-15, -3)
#' upper <- c(-5, 3)
#' opt <- pso(fn = fn, 
#'            pop.initial = c(8, 2),
#'            lower = lower, upper = upper,
#'            is.max = FALSE)
#' print(opt$par)
#' 
pso <-  function(fn, 
                 gr = NULL, 
                 lower, 
                 upper,
                 pop.n = 40,
                 pop.initial = NULL,
                 pop.method = "uniform",
                 nh.method = "random",
                 nh.par = 3,
                 nh.adaptive = TRUE,
                 velocity.method = "hypersphere",
                 velocity.par = list(w = 1 / (2 * log(2)), 
                                     c1 = 0.5 + log(2),
                                     c2 = 0.5 + log(2)),
                 hybrid.method = "rank",
                 hybrid.par = 2,
                 hybrid.prob = 0,
                 hybrid.opt.par = NULL,
                 hybrid.n = 1,
                 constr.method = NULL,
                 constr.par = NULL,
                 random.order = TRUE,
                 maxiter = 100,
                 is.max = TRUE,
                 info = TRUE,
                 ...)
{
  # Get the parameters to be passed to fitness
  
  dots <- list(...)
  
  # Initialize some variables
  
  genes.n <- length(lower)
  
  # Perform initial validation
  
    # fn
  
  if (!is.function(fn))
  {
    stop("Please, ensure that 'fn' is a function.\n")
  }
  
    # gr
  if(!is.null(gr))
  {
    if (!is.function(gr))
    {
      stop("Please, ensure that 'gr' is a function.\n")
    }
  }
  
    # lower and upper
  
  if (length(lower) != length(upper))
  {
    stop("Please, ensure that 'lower' and 'upper' are of the same length.\n")
  }
  
  if (any(lower > upper))
  {
    stop(paste0("Please, ensure that all elements of the 'lower' are ",
                "smaller than the corresponding elements of the 'upper.'",
                "\n"))
  }
  
  lower[is.infinite(lower)] <- -(.Machine$double.xmax * 0.9)
  upper[is.infinite(upper)] <- .Machine$double.xmax * 0.9
  
    # pop.initial
  
  if (!is.null(pop.initial))
  {
    if (!is.numeric(pop.initial))
    {
      stop("Please, ensure that 'pop.initial' is a numeric matrix.\n")
    }
    
    if (!is.matrix(pop.initial))
    {
      pop.initial <- matrix(pop.initial, nrow = 1)
    }
    
    if (ncol(pop.initial) != genes.n)
    {
      stop(paste0("Please, ensure that the number of columns of 'pop.initial' ",
                  "equals the number of estimated parameters, i.e., genes.",
                  "\n"))
    }
  }
  
    # pop.n
  
  if (pop.n <= 1)
  {
    stop("Please, ensure that 'pop.n' is an integer greater than one.")
  }
  
  if ((pop.n %% 2) != 0) 
  {
    pop.n <- pop.n + 1
    warning("Since 'pop.n' is not even, it has been incremented by one.\n")
  }
  
  # Initialize some additional variables  
  
  fitness <- rep(NA, pop.n)
  
  # Continue validation
  
    # hybrid.method and hybrid.par
  
  if (!is.function(hybrid.method))
  {
    hybrid.par <- gena.mating.validate(method = hybrid.method,
                                       par = hybrid.par,
                                       parents.n = pop.n)
  }
  else
  {
    gena.hybrid <- hybrid.method
  }
  
    # hybrid.prob
  
  if (is.numeric(hybrid.prob))
  {
    if ((hybrid.prob < 0) | (hybrid.prob > 1))
    {
      stop("Please, ensure that 'hybrid.prob' is between 0 and 1.\n")
    }
  } else {
    stop("Please, ensure that 'hybrid.prob' is a numeric value.\n")
  }
  
    # hybrid.n
  
  if (hybrid.n < 0)
  {
    stop("Please, ensure that 'hybrid.n' is a non-negative integer.\n")
  }
  
  if (hybrid.n > pop.n)
  {
    stop("Please, ensure that 'hybrid.n' is not greater than 'pop.n'.")
  }
  
    # maxiter
  
  if (maxiter < 1)
  {
    stop("Please, ensure that 'maxiter' is a positive integer.")
  }
  
    # constr.method and constr.par
  
  if(!is.null(constr.method))
  {
    if (!(is.function(constr.method) | is.character(constr.method)))
    {
      stop("Please, ensure that 'constr.method' is a function or character.\n")
    }
    if (is.character(constr.method))
    {
      if ((constr.method == "bounds") & is.null(constr.par))
      {
        constr.par$lower <- lower
        constr.par$upper <- upper
      }
      gena.constr.validate(method = constr.method, par = constr.par)
    }
    if (is.function(constr.method))
    {
      gena.constr <- constr.method
    }
  }
  
    # is.max
  
  if (!is.logical(info))
  {
    if ((info == 0) | (info == 1))
    {
      info <- as.logical(info)
    } else {
      stop("Please, ensure that 'is.max' is a logical.\n")
    }
  }
  
    # info
  
  if (!is.logical(info))
  {
    if ((info == 0) | (info == 1))
    {
      info <- as.logical(info)
    } else {
      stop("Please, ensure that 'info' is a logical.\n")
    }
  }
  
    # random.order
  
  if (!is.logical(random.order))
  {
    if ((random.order == 0) | (random.order == 1))
    {
      random.order <- as.logical(random.order)
    } else {
      stop("Please, ensure that 'random.order' is a logical.\n")
    }
  }
  
    # nh.adaptive
  
  if (!is.logical(nh.adaptive))
  {
    if ((nh.adaptive == 0) | (nh.adaptive == 1))
    {
      nh.adaptive <- as.logical(nh.adaptive)
    } else {
      stop("Please, ensure that 'nh.adaptive' is a logical.\n")
    }
  }
  
    # nh.method and nh.par
  
  if (!is.function(nh.method))
  {
    nh.par <- pso.nh.validate(method = nh.method,
                              par = nh.par,
                              pop.n = pop.n)
  }
  else
  {
    pso.nh <- nh.method
  }
  
    # velocity.method and velocity.par
  
  if (!is.function(velocity.method))
  {
    velocity.par <- pso.velocity.validate(method = velocity.method,
                                          par = velocity.par)
  }
  else
  {
    pso.velocity <- velocity.method
  }
  
  # Value to reverse signs of fitnesses if need
  
  is.max.val <- 2 * is.max - 1
  
  # Deal with hybrid optimizer default parameters
  
  hybrid.opt.par.default = list(fn = fn,
                                gr = gr,
                                method = ifelse(is.null(gr), 
                                                "Nelder-Mead", 
                                                "BFGS"),
                                control = list(fnscale = -1 * is.max.val,
                                               abstol = 1e-10,
                                               reltol = 1e-10,
                                               maxit = 1000000000))
  
  if (!is.null(hybrid.opt.par))
  {
    if (!is.null(hybrid.opt.par$control))
    {
      hybrid.opt.par.default$control <- modifyList(hybrid.opt.par.default$control,
                                                   hybrid.opt.par$control)
    }
    hybrid.opt.par <- modifyList(hybrid.opt.par.default, 
                                 hybrid.opt.par)
  } else {
    hybrid.opt.par <- hybrid.opt.par.default
  }
  
  # Get the name of the first argument of the function
  
  fn.par <- names(formals(fn))[1]
  
  # Number of function and gradient evaluations
  
  counts <- c(0, 0)
  names(counts) <- c("function", "gradient")
  
  # -------------------------------------
  # Perform preliminary iteration of PSO
  # -------------------------------------
  
  # Vector of population members indices
  
  pop.ind <- 1:pop.n
  
  # Define neighbourhoods of particles
  
  nh <- pso.nh(pop.n = pop.n,
               method = nh.method,
               par = nh.par,
               iter = 0)

  # Initialize the population
  
  if (is.function(pop.method))
  {
    gena.population <- pop.method
  }
  
  population <- gena.population(pop.n = pop.n,
                                lower = lower, 
                                upper = upper,
                                method = pop.method,
                                pop.initial = pop.initial)
  
  # Calculate initial fitnesses
  
  fitness <- rep(NA, pop.n)
  for(i in 1:pop.n)
  {
    dots[[fn.par]] <- population[i, ] 
    fitness[i] <- do.call(what = fn, args = dots)
  }
  fitness[is.na(fitness)] <- -Inf
  
  # Adjust for minimization if needed
  
  if (!is.max)
  {
    fitness <- -fitness
  }

  # Assign best personal positions
  
  best.pn <- population
  best.pn.fitness <- fitness
  
  # Assign best positions in the neighbourhood
  
  best.nh <- matrix(NA, nrow = pop.n, ncol = genes.n)
  best.nh.fitness <- rep(NA, pop.n)
  for (j in 1:pop.n)
  {
    best.nh.ind <- which.max(best.pn.fitness[nh[[j]]])
    best.nh[j, ] <- population[nh[[j]], , drop = FALSE][best.nh.ind, ]
    best.nh.fitness[j] <- best.pn.fitness[nh[[j]]][best.nh.ind]
  }

  # Calculate initial velocities of the particles
  
  velocity <- matrix(NA, nrow = pop.n, ncol = genes.n)
  for (j in 1:genes.n)
  {
    runif.tmp <- runif(pop.n)
    velocity[, j] <- runif.tmp * lower[j] + (1 - runif.tmp) * upper[j] - 
                     population[, j]
  }

  # Store information on the best particle
  
  best.final.ind <- which.max(best.pn.fitness)
  best.final <- best.pn[best.final.ind, ]
  best.final.fitness <- best.pn.fitness[best.final.ind]
  
  # Historical best fitness values
  
  fitness.history <- NULL

  # -------------------------------------
  # Perform the particle swarm algorithm
  # -------------------------------------
  
  for(i in 1:maxiter)
  {
    # Get the parameters to be passed to fitness
    
    dots <- list(...)
    
    # Randomize the order of particles
    
    if (random.order)
    {
      pop.ind <- sample(x = pop.ind, size = pop.n, replace = FALSE)
    }
    
    # Impose the constraints on the parameters
    
    constr.ind <- NULL
    if (!is.null(constr.method))
    {
      if (is.function(constr.method))
      {
        constr.par$population <- population
        constr.par$iter <- i
        constr.list <- do.call(what = constr.method, 
                               args = constr.par)
        population <- constr.list$population
        constr.ind <- constr.list$constr.ind
      }
      else
      {
        constr.list <- gena.constr(population = population, 
                                   iter = i,
                                   method = constr.method,
                                   par = constr.par)
        population <- constr.list$population
        constr.ind <-constr.list$constr.ind
      }
    }
    if (!is.null(constr.ind))
    {
      velocity[constr.ind] <- -0.5 * velocity[constr.ind]
    }
    
    # Update velocities and position
    
    velocity <- pso.velocity(population = population,
                             velocity = velocity,
                             method = velocity.method,
                             par = velocity.par,
                             best.pn = best.pn,
                             best.nh = best.nh,
                             best.pn.fitness = best.pn.fitness, 
                             best.nh.fitness = best.nh.fitness,
                             iter = i)
    population <- population + velocity
    
    # Estimate the fitness of each particle
    
    for(j in pop.ind)
    {
      dots[[fn.par]] <- population[j, ] 
      fitness[j] <- do.call(what = fn, args = dots)
    }
    fitness[is.na(fitness) | is.nan(fitness)] <- -Inf
    
    # Make the hybrids
    
    hybrid.value <- runif(1, 0, 1)
    
    if (((hybrid.value <= hybrid.prob) |
         (i == 1) | (i == maxiter)) & 
        (hybrid.n >= 1) & (hybrid.prob > 0))
    {
      hybrids.list <- gena.hybrid(population = population,
                                  fitness = fitness,
                                  hybrid.n = hybrid.n,
                                  method = hybrid.method,
                                  par = hybrid.par,
                                  opt.par = hybrid.opt.par,
                                  info = info,
                                  ... = ...)
      population <- hybrids.list$population
      fitness <- hybrids.list$fitness
      counts <- hybrids.list$counts + counts
      # velocity[hybrids.list$hybrids.ind, ] <- rep(0, genes.n)
    }
    
    # Reverse signs of fitnesses for minimization task
    
    if (!is.max)
    {
      fitness <- -fitness
    }
    
    # Update best local position
  
    is_better <- fitness > best.pn.fitness
    best.pn[is_better, ] <- population[is_better, ]
    best.pn.fitness[is_better] <- fitness[is_better]

    # Update best global position

    for (j in pop.ind)
    {
      best.nh.ind <- which.max(best.pn.fitness[nh[[j]]])
      candidate.fitness <- best.pn.fitness[nh[[j]]][best.nh.ind]
      if (candidate.fitness > best.nh.fitness[j])
      {
        best.nh[j, ] <- best.pn[nh[[j]], , drop = FALSE][best.nh.ind, ]
        best.nh.fitness[j] <- candidate.fitness
      }
    }
    
    # Update the best particle
    
    best.old.fitness <- best.final.fitness
    best.final.ind <- which.max(best.pn.fitness)
    best.final <- best.pn[best.final.ind, ]
    best.final.fitness <- best.pn.fitness[best.final.ind]
    
    # Store fitness of the best particle
    
    fitness.history <- c(fitness.history, best.final.fitness)

    # Change the topology if needed, along with best
    # known solution in the neighbourhood
    
    if (nh.adaptive & (best.old.fitness >= best.final.fitness))
    {
      # Update the neighbourhood
      
      nh <- pso.nh(pop.n = pop.n,
                   method = nh.method,
                   par = nh.par,
                   iter = i)
      
      # Update best known solutions in the neighbourhood
      
      for (j in pop.ind)
      {
        best.nh.ind <- which.max(best.pn.fitness[nh[[j]]])
        best.nh[j, ] <- best.pn[nh[[j]], , drop = FALSE][best.nh.ind, ]
        best.nh.fitness[j] <- best.pn.fitness[nh[[j]]][best.nh.ind]
      }
    }
    
    if (info)
    {
      cat(paste0("Iter = ", i, ", ",
                 "mean = ", is.max.val * signif(mean(fitness), 5), ", ",
                 "median = ", is.max.val * signif(median(fitness), 5), ", ",
                 "best = ", is.max.val * signif(best.final.fitness, 5), "\n"))
    }

    # Return the results if needed
    
    if (i == maxiter)
    {
      counts[1] <- counts[1] + i * pop.n
      return.list <- list(population = population,
                          par = is.max.val * best.final,
                          value = is.max.val * best.final.fitness,
                          counts = counts,
                          fitness.history = is.max.val * fitness.history,
                          is.max = is.max,
                          iter = i)
      class(return.list) <- "pso"
      return(return.list)
    }
    
  }
  
  return(NULL)
}

#' Print method for "pso" object
#' @param x Object of class "pso"
#' @param ... further arguments (currently ignored)
#' @returns This function does not return anything.
print.pso <- function(x, ...)
{
  cat("We have optimized, optimized, and finally optimized! \n")
  cat("---\n")
  cat(paste0(ifelse(x$is.max, "The largest ", "The smallest "),
             "value of the fitness function found is ", 
             signif(x$value, 5), " at point: \n"))
  cat(signif(x$par, 5))
  cat("\n")
  cat("---\n")
  cat(paste0("There were ", x$counts[1], " calls to the function and ",
             x$counts[2], " calls to the gradient. \n"))
  cat("---\n")
  cat("This object is a list containing the following elements:\n")
  cat(names(x))
}

#' Summarizing pso Fits
#' @param object Object of class "pso"
#' @param ... further arguments (currently ignored)
#' @return This function returns the same list as \code{\link[gena]{pso}} 
#' function changing its class to "summary.pso".
summary.pso <- function(object, ...)
{
  if (length(list(...)) > 0)
  {
    warning("Additional arguments passed through ... are ignored.")   
  }
  class(object) <- "summary.pso"
  return(object)
}

#' Summary for "pso" object
#' @param x Object of class "pso"
#' @param ... further arguments (currently ignored)
#' @returns This function returns \code{x} input argument.
print.summary.pso <- function (x, ...) 
{
  if (length(list(...)) > 0)
  {
    warning("Additional arguments passed through ... are ignored.")   
  }
  print.pso(x)
  return(x)
}

#' Plot best found fitnesses during genetic algorithm
#' @param x Object of class "pso"
#' @param y this parameter currently ignored
#' @param ... further arguments (currently ignored)
#' @returns This function does not return anything.
plot.pso <- function (x, y = NULL, ...) 
{
  plot(x = 1:x$iter, y = x$fitness.history, 
       main = "Historical values of fitness", 
       xlab = "Iteration", 
       ylab = paste0(ifelse(x$is.max, "The largest ", "The smallest "),
                     "fitness value"))
}

#' Numeric Differentiation
#' @name genaDiff
#' @description Numeric estimation of the gradient and Hessian.
#' @param fn function for which gradient or Hessian should be calculated.
#' @param gr gradient function of \code{fn}.
#' @param par point (parameter values) at which \code{fn} should be 
#' differentiated.
#' @param eps numeric vector representing the increment of the \code{par}. 
#' So \code{eps[i]} represents the increment of \code{par[i]}. If \code{eps} is
#' a constant, then all increments are the same.
#' @param method numeric differentiation method: "central-difference" or
#' "forward-difference".
#' @param fn.args list containing arguments of \code{fn} except \code{par}.
#' @param gr.args list containing arguments of \code{gr} except \code{par}.
#' @param is.gc logical; if \code{TRUE}, then garbage collection is called 
#' inside loops to reduce memory usage.
#' @details It is possible to substantially improve numeric Hessian accuracy 
#' by using analytical gradient \code{gr}. If both \code{fn} and \code{gr}
#' are provided then only \code{gr} will be used. If only \code{fn} is provided 
#' for \code{gena.hessian} then \code{eps} will be transformed to 
#' \code{sqrt(eps)} for numeric stability purposes.
#'
#' @return Function \code{gena.grad} returns a vector that is the gradient of 
#' \code{fn} at the point \code{par} calculated via the \code{method} numeric 
#' differentiation approach using the increment \code{eps}.
#' 
#' Function \code{gena.hessian} returns a matrix that is the Hessian of 
#' \code{fn} at the point \code{par}.
#' @examples 
#' # Consider the following function
#' fn <- function(par, a = 1, b = 2)
#' {
#'   val <- par[1] * par[2] - a * par[1] ^ 2 - b * par[2] ^ 2
#' }
#' 
#' # Calculate the gradient at point (2, 5) with respect to 'par' 
#' # when 'a = 1' and 'b = 1'
#' par <- c(2, 5)
#' fn.args = list(a = 1, b = 1)
#' gena.grad(fn = fn, par = par, fn.args = fn.args)
#' 
#' # Calculate Hessian at the same point
#' gena.hessian(fn = fn, par = par, fn.args = fn.args)
#' 
#' # Repeat calculation of the Hessian using analytical gradient
#' gr <- function(par, a = 1, b = 2)
#' {
#'   val <- c(par[2] - 2 * a * par[1],
#'            par[1] - 2 * b * par[2])
#' }
#' gena.hessian(gr = gr, par = par, gr.args = fn.args)
#' 
#' @rdname genaDiff
#' @export
gena.grad <- function(fn, par, 
                      eps = sqrt(.Machine$double.eps) * abs(par),
                      method = "central-difference",
                      fn.args = NULL)
{
  n_par <- length(par)
  
  if(length(eps) == 1)
  {
    eps <- rep(eps, n_par)
  }
    
  par_plus <- par
  par_minus <- par
    
  val <- rep(0, n_par)

  for(i in 1:n_par)
  {
    if (method == "central-difference")
    {
      par_plus[i] <- par[i] + eps[i]
    }

    par_minus[i] <- par[i] - eps[i]
      
    fn.args$par <- par_plus
    fn_plus <- do.call(fn, fn.args)
   
    fn.args$par <- par_minus
    fn_minus <- do.call(fn, fn.args)
      
    par_plus <- par
    par_minus <- par
       
    if (method == "central-difference")
    {
      val[i] <- (fn_plus - fn_minus) / (2 * eps[i])
    }
    if (method == "forward-difference")
    {
      val[i] <- (fn_plus - fn_minus) / eps[i]
    }
  }
  
  return(val)
}

#' @rdname genaDiff
#' @export
gena.hessian <- function(fn = NULL, gr = NULL, 
                         par, eps = sqrt(.Machine$double.eps) * abs(par), 
                         fn.args = NULL, gr.args = NULL, is.gc = FALSE)
{
  n_par <- length(par)
  
  if (is.null(gr))
  {
    eps <- sqrt(eps)
  }
  
  if(length(eps) == 1)
  {
    eps <- rep(eps, n_par)
  }
  
  par_plus <- par
  par_minus <- par
  
  val <- matrix(0, n_par, n_par)
  
  # With analytical gradient
  if (!is.null(gr))
  {
    for(i in 1:n_par)
    {
      par_plus[i] <- par[i] + eps[i]
      par_minus[i] <- par[i] - eps[i]
        
      gr.args$par <- par_plus
      gr_plus <- do.call(gr, gr.args)
        
      gr.args$par <- par_minus
      gr_minus <- do.call(gr, gr.args)
        
      par_plus <- par
      par_minus <- par
        
      val[i, ] <- (gr_plus - gr_minus) / (2 * eps[i])
      
      if (is.gc)
      {
        gc()
      }
    }
    
    for (i in 1:n_par)
    {
      for (j in 1:i)
      {
        if (i != j)
        {
          val[i, j] <- (val[i, j] + val[j, i]) / 2
          val[j, i] <- val[i, j]
        }
      }
    }
  }
  # Without analytical gradient
  else
  {
    fn.args$par <- par
    fn_0 <- do.call(fn, fn.args)
    
    fn_plus <- rep(NA, n_par)
    for (i in 1:n_par)
    {
      par_plus[i] <- par[i] + eps[i]
      fn.args$par <- par_plus
      fn_plus[i] <- do.call(fn, fn.args)
      par_plus[i] <- par[i]
      
      if (is.gc)
      {
        gc()
      }
    }
    
    par_ij <- par
    for (i in 1:n_par)
    {
      par_ij[i] <- par[i] + eps[i]
      for (j in 1:i)
      {
        par_ij[j] <- par_ij[j] + eps[j]
        fn.args$par <- par_ij
        
        fn_ij <- do.call(fn, fn.args)
        val[i, j] <- (fn_ij - fn_plus[i] - fn_plus[j] + fn_0) / 
                     (eps[i] * eps[j])
        val[j, i] <- val[i, j]
          
        par_ij[j] <- par[j]
      }
      par_ij[i] <- par[i]
      
      if (is.gc)
      {
        gc()
      }
    }
  }
  
  return(val)
}

gena.lineSearch <- function(fn, gr = NULL,
                            fn.val, gr.val,
                            par,
                            dir, lr = 1,
                            c1 = 1e-4, c2 = 0.9,
                            mult = 0.7,
                            maxiter = 100,
                            fn.args = list(), gr.args = list())
{
  # Variable to control whether learning rate is increasing
  is_decrease = TRUE
  
  # Learning rate values
  lr.vec <- rep(NA, maxiter)
  lr.vec[1] <- lr
  
  # Parameter values
  par.mat <- matrix(NA, ncol = length(par), nrow = maxiter)
  
  # Function and gradient calls
  fn.calls <- 0
  gr.calls <- 0
  
  # Non-monotone line search
  # M <- min(length(fn.val), 5)
  # fn.val <- mean(tail(fn.val, M))
  
  # Wolfe's conditions
  armajilo <- FALSE
  curvature <- FALSE
  
  for (i in 1:maxiter)
  {
    # Calculate new point and function
    # value at this point
    par.mat[i, ] <- par + lr.vec[i] * dir
    fn.args$par <- par.mat[i, ]
    fn.val.new <- do.call(fn, fn.args)
    fn.calls <- fn.calls + 1
    
    # Check Armijo condition
    gr.dir <- sum(gr.val * dir)
    armajilo <- fn.val.new <= (fn.val + c1 * lr.vec[i] * gr.dir)
    if (armajilo)
    {
      # Check strong Curvature condition
      gr.args$par <- par.mat[i, ]
      gr.val.new <- NULL
      if(!is.null(gr))
      {
        gr.val.new <- matrix(do.call(gr, gr.args), ncol = 1)
      }
      else
      {
        gr.val.new <- gena.grad(fn = fn, par = par.mat[i, ], 
                                fn.args = fn.args)
      }
      gr.calls <- gr.calls + 1
      gr.div.new <- sum(gr.val.new * dir)
      curvature <- gr.div.new <= (c2 * gr.dir)
      if (curvature)
      {
        if (is_decrease)
        {
          if (i == 1)
          {
            is_decrease <- FALSE
          }
          else
          {
            return.list <- list(lr = lr.vec[i],
                                par = par.mat[i, ],
                                fail = FALSE,
                                fn.calls = fn.calls,
                                gr.calls = gr.calls)
            return(return.list)
          }
        }
      }
    }
    if(!(armajilo & curvature) & !is_decrease)
    {
      return.list <- list(lr = lr.vec[i - 1],
                          par = par.mat[i - 1, ],
                          fail = FALSE,
                          fn.calls = fn.calls,
                          gr.calls = gr.calls)
      return(return.list)
    }
    if (is_decrease)
    {
      lr.vec[i + 1] <- lr.vec[i] * mult
    }
    else
    {
      lr.vec[i + 1] <- lr.vec[i] / mult
    }
  }
  
  # If line search is not successful
  return.list <- list(lr = 0,
                      par = par,
                      fail = TRUE,
                      fn.calls = fn.calls,
                      gr.calls = gr.calls)

  return(return.list)
}

gena.local <- function(par,                                   # initial point
                       fn,                                    # function to optimize
                       gr = NULL,                             # gradient of the function
                       maxiter = 1e+6,                        # the number of iterations
                       reltol = 1e-10,                        # relative tolerance
                       abstol = 1e-10,                        # absolute tolerance
                       lr = 1024,                             # learning rate
                       type = "BFGS",                         # optimization type
                       ...)                                   # additional arguments for fn and gr
{
  dots <- list(...)
  iter <- NULL
  dir <- NULL
  
  fn.calls <- 0
  gr.calls <- 0
  
  reltol.cond <- FALSE
  maxiter.cond <- FALSE
  abstol.cond <- FALSE

  # Get the number of estimated parameters
  n_x <- length(par)

  # Initialize the matrix to store the
  # points for each iteration
  x <- matrix(NA,
              nrow = maxiter,
              ncol = length(par))
  x[1, ] <- par

  # Initialize variable to store function 
  # value and gradient information
  fn.val <- rep(NA, n_x)
  gr.val <- matrix(NA, nrow = maxiter, ncol = n_x)
  
  # Initialize variable to store Hessians and their inverse
  hessian <- array(dim = c(maxiter + 1, n_x, n_x))
  hessian.inv <- hessian
  hessian[1, , ] <- diag(rep(1, n_x)) #solve(gena.hessian(fn = fn, gr = gr, par = par))
  hessian.inv[1, , ] <- hessian[1, , ]
  
  # Estimate gradient and function value
  # at the initial point
  dots$par <- par
  fn.val[1] <- do.call(fn, dots)
  if(is.null(gr))
  {
    gr.val[1, ] <- gena.grad(fn, x[1, ], fn.args = dots)
  } else {
    dots$par <- x[1, ]
    gr.val[1, ] <- do.call(gr, dots)
  }
  gr.calls <- gr.calls + 1

  # Start the algorithm
  for (i in 1:maxiter)
  {
    # Determine the direction
    dir <- -hessian.inv[i, , ] %*% matrix(gr.val[i, ], ncol = 1)

    # Perform the line search
    ls.result <- gena.lineSearch(fn = fn, gr = gr,
                                 fn.val = fn.val[i], 
                                 gr.val = gr.val[i, ],
                                 par = x[i, ],
                                 dir = dir, lr = lr,
                                 fn.args = dots,
                                 gr.args = dots)
    # if (ls.result$fail)
    # {
    #   print(123)
    #   ls.result <- gena.lineSearch(fn = fn, gr = gr,
    #                                fn.val = fn.val[i], 
    #                                gr.val = gr.val[i, ],
    #                                par = x[i, ],
    #                                dir = -dir, lr = lr,
    #                                fn.args = dots,
    #                                gr.args = dots)
    #   hessian.inv[i, , ] <- diag(rep(1, n_x))
    # }
    x[i + 1, ] <- ls.result$par
    lr <- ls.result$lr
    fn.calls <- fn.calls + ls.result$fn.calls
    gr.calls <- gr.calls + ls.result$gr.calls

    # Estimate the function value at new point
    dots$par <- x[i + 1, ]
    fn.val[i + 1] <- do.call(fn, dots)
    fn.calls <- fn.calls + 1
    print(c(fn.val[i], fn.val[i + 1] , lr))
    
    # Estimate the gradient and new point
    if(is.null(gr))
    {
      gr.val[i + 1, ] <- gena.grad(fn, x[i + 1, ], fn.args = dots)
    } else {
      dots$par <- x[i + 1, ]
      gr.val[i + 1, ] <- do.call(gr, dots)
    }

    # Calculate inverse of the Hessian
    y <- matrix(gr.val[i + 1, ] - gr.val[i, ], ncol = 1)
    s <- matrix(lr * dir, ncol = 1)
    
    hessian.inv[i + 1, , ] <- hessian.inv[i, , ] +
                              (s %*% t(s)) * 
                              (as.vector(t(s) %*% y + 
                                         t(y) %*% hessian.inv[i, , ] %*% y) /
                              as.vector((t(s) %*% y) ^ 2)) -
                              (hessian.inv[i, , ] %*% y %*% t(s) + 
                               s %*% t(y) %*% hessian.inv[i, , ]) / 
                              as.vector(t(s) %*% y)
    
    # I <- diag(rep(1, n_x))
    # hessian.inv[i + 1, ,] <- (I - s %*% t(y) / as.vector(t(y) %*% s)) %*% 
    #                          hessian.inv[i, , ] %*%
    #                          (I - y %*% t(s) / as.vector(t(y) %*% s)) +
    #                          (s %*% t(s)) / as.vector(t(y) %*% s)

    # Print the information
    cat(paste0("iter-", i, ": ", fn.val[i], "\n"))

    # Check whether termination conditions
    # are satisfied
    abstol.cond <- abs(fn.val[i] - fn.val[i + 1]) < abstol
    if (fn.val[i] != 0)
    {
      reltol.cond <- all(abs((fn.val[i] - fn.val[i + 1]) /
                              fn.val[i]) < reltol)
    }
    else
    {
      reltol.cond <- FALSE
    }
    maxiter.cond <- i == maxiter
    if (abstol.cond | reltol.cond | maxiter.cond)
    {
      iter <- i
      break
    }
  }

  # Get the final point
  return_list <- list(points = x[1:iter],
                      solution = x[iter, ],
                      value = fn.val[iter],
                      fn_grad = gr.val[iter, ],
                      hessian = solve(hessian.inv[iter, , ]),
                      hessian.inv = hessian.inv[iter, , ],
                      fn.calls = fn.calls,
                      gr.calls = gr.calls,
                      dir = dir)

  return(return_list)
}

#' Mating
#' @description Mating (selection) method (algorithm) to be used in the
#' genetic algorithm.
#' @param population numeric matrix whose rows are chromosomes, i.e., vectors of 
#' parameter values.
#' @param fitness numeric vector whose \code{i}-th element is the value of 
#' \code{fn} at point \code{population[i, ]}.
#' @param parents.n even positive integer representing the number of parents.
#' @param method mating method to be used for selection of parents.
#' @param par additional parameters to be passed depending on the \code{method}.
#' @param self logical; if \code{TRUE} then a chromosome may mate with itself. 
#' Otherwise, mating is allowed only between different chromosomes.
#' @param iter iteration number of the genetic algorithm.
#' @details Denote \code{population} by \eqn{C} whose \code{i}-th row 
#' \code{population[i, ]} is a chromosome \eqn{c_{i}}, i.e., the vector of 
#' parameter values of the function being optimized \eqn{f(.)} that is
#' provided via the \code{fn} argument of \code{\link[gena]{gena}}.
#' The elements of chromosome \eqn{c_{ij}} are genes representing parameters 
#' values. Argument \code{fitness} is a vector of function values at
#' corresponding chromosomes i.e. \code{fitness[i]} corresponds to
#' \eqn{f_{i}=f(c_{i})}. Total number of chromosomes in population
#' \eqn{n_{population}} equals to \code{nrow(population)}.
#' 
#' The mating algorithm determines the selection of chromosomes that will become 
#' parents. During each iteration of mating, one of the chromosomes becomes a 
#' parent until there are \eqn{n_{parents}} (i.e., \code{parents.n}) parents 
#' selected. Each chromosome may become a parent multiple times or not become a 
#' parent at all.
#' 
#' Denote by \eqn{c^{s}_{i}} the \eqn{i}-th selected parent. Parents
#' \eqn{c^{s}_{i}} and \eqn{c^{s}_{i + 1}} form a pair that will further
#' produce a child (offspring), where \eqn{i} is odd.
#' If \code{self = FALSE}, then for each pair of parents 
#' \eqn{(c_{i}^s, c_{i+1}^s)} it is ensured that 
#' \eqn{c^{s}_{i} \ne c^{s}_{i + 1}} except in the case when there are several 
#' identical chromosomes in the population. However, \code{self} is ignored 
#' if \code{method} is \code{"tournament"}, so in this case self-mating
#' is always possible.
#' 
#' Denote by \eqn{p_{i}} the probability of a chromosome becoming a parent.
#' Note that each chromosome may become a parent multiple times.
#' The probability \eqn{p_{i}\left(f_{i}\right)} is a function 
#' of fitness \eqn{f_{i}}. Usually this function is non-decreasing, so 
#' fitter chromosomes have a higher probability of becoming a parent. 
#' There is also an intermediate value \eqn{w_{i}} called the weight such that:
#' \deqn{p_{i}=\frac{w_{i}}{\sum\limits_{j=1}^{n_{population}}w_{j}}}
#' Therefore, all weights \eqn{w_{i}} are proportional to the corresponding 
#' probabilities \eqn{p_{i}} by the same factor (the sum of weights).
#' 
#' Argument \code{method} determines particular mating algorithm to be applied.
#' Denote by \eqn{\tau} the vector of parameters used by the algorithm.
#' Note that \eqn{\tau} corresponds to \code{par}. The algorithm determines
#' a particular form of the \eqn{w_{i}\left(f_{i}\right)} function which 
#' in turn determines \eqn{p_{i}\left(f_{i}\right)}.
#' 
#' If \code{method = "constant"}, then all weights and probabilities are equal:
#' \deqn{w_{i}=1 => p_{i}=\frac{1}{n_{population}}}
#' 
#' If \code{method = "rank"}, then each chromosome receives a rank \eqn{r_{i}} 
#' based on the fitness \eqn{f_{i}} value. So if \code{j}-th chromosome is the
#' fittest one and \code{k}-th chromosome has the lowest fitness value, then
#' \eqn{r_{j}=n_{population}} and  \eqn{r_{k}=1}. The relationship
#' between weight \eqn{w_{i}} and rank \eqn{r_{i}} is as follows:
#' \deqn{w_{i}=\left(\frac{r_{i}}{n_{population}}\right)^{\tau_{1}}}
#' The greater value of \eqn{\tau_{1}} the greater portion of probability will
#' be delivered to fitter chromosomes. 
#' Default value is \eqn{\tau_{1} = 0.5} so \code{par = 0.5}.
#' 
#' If \code{method = "fitness"} then weights are calculated as follows:
#' \deqn{w_{i}=\left(f_{i} - 
#'                   \min\left(f_{1},...,f_{n_{population}}\right) + 
#'                   \tau_{1}\right)^{\tau_{2}}}
#' By default \eqn{\tau_{1}=10} and \eqn{\tau_{2}=0.5}, i.e., 
#' \code{par = c(10, 0.5)}. There is a restriction \eqn{\tau_{1}\geq0}
#' ensuring that the expression in brackets is non-negative.
#' 
#' If \code{method = "tournament"} then  \eqn{\tau_{1}} (i.e., \code{par}) 
#' chromosomes will be randomly selected with equal probabilities and without 
#' replacement. Then the chromosome with the highest fitness 
#' (among these selected chromosomes) will become a parent.
#' It is possible to provide the representation of this algorithm via 
#' probabilities \eqn{p_{i}} but the formulas are numerically unstable.
#' By default \code{par = min(5, ceiling(parents.n * 0.1))}.
#' 
#' Validation and default values assignment for \code{par} are performed inside
#' the \code{\link[gena]{gena}} function not in \code{\link[gena]{gena.mating}}.
#' This allows validation to be performed a single time instead of repeating it
#' each iteration of genetic algorithm.
#' 
#' For more information on mating (selection) algorithms,
#' please see Shukla et al. (2015).
#' 
#' @return The function returns a list with the following elements:
#' \itemize{
#' \item \code{parents} - matrix whose rows are parents. The number of
#' rows of this matrix equals to \code{parents.n} while the number of columns
#' is \code{ncol(population)}.
#' \item \code{fitness} - vector whose i-th element is the fitness of the
#' i-th parent.
#' \item \code{ind} - vector whose i-th element is the index of the i-th
#' parent in the population, so \code{$parents[i, ]} equals 
#' \code{population[ind[i], ]}.
#' }
#' @references A. Shukla, H. Pandey, D. Mehrotra (2015). 
#' Comparative review of selection techniques in genetic algorithm.
#' \emph{2015 International Conference on Futuristic Trends on Computational 
#' Analysis and Knowledge Management (ABLAZE)}, 515-519,
#' <doi:10.1109/ABLAZE.2015.7154916>.
#' @examples 
#' # Consider the following fitness function
#' fn <- function(x)
#' {
#'   val <- x[1] * x[2] - x[1] ^ 2 - x[2] ^ 2
#' }
#' 
#' # Randomly initialize the population
#' set.seed(123)
#' n.pop <- 10
#' population <- gena.population(pop.n = n.pop,
#'                               lower = c(-5, -5), 
#'                               upper = c(5, 5))
#'
#' # Calculate fitness of each chromosome
#' fitness <- rep(NA, n.pop)
#' for(i in 1:n.pop)
#' {
#'   fitness[i] <- fn(population[i, ])
#' }
#' 
#' # Perform mating to select parents
#' parents <- gena.mating(population = population,
#'                        fitness = fitness,
#'                        parents.n = n.pop,
#'                        method = "rank",
#'                        par = 0.8)
#' print(parents)
#' 
gena.mating <- function(population,                        # population of chromosomes
                        fitness,                           # fitness values of chromosomes
                        parents.n,                         # number of parents to select
                        method = "rank",                   # algorithm to assign probabilities 
                                                           # of being a parent
                        par = NULL,                        # additional parameters
                                                           # depending on "method"
                        self = FALSE,
                        iter = NULL)                       # genetic algorithm iteration              
{      
  # ---
  # The algorithm brief description:
  # 1. For each chromosome (solution) estimate the 
  #    probability of being selected to be a parent.
  # 2. Select the parents according to estimated
  #    probabilities or some other criteria.
  # 3. Return the parents along with their fitness.
  # ---
  
  # Prepare some values
  
  pop.n <- nrow(population)                                # the number of chromosomes
  ind_pop <- 1:pop.n                                       # chromosomes indexes
  
  # Estimate the probability of being a parent
  # for each chromosome
  
  probs <- rep(1 / pop.n, pop.n)                           # default probabilities vector
  
  if (method == "rank")                                    # rank-based probabilities
  {
    fitness_rank <- rank(fitness)                          # rank by fitness
    fitness_weight <- (fitness_rank / pop.n) ^ par[1]      # weight according to rank
    probs <- fitness_weight / sum(fitness_weight)          # assign probabilities
                                                           # proportional to weights
  }
  
  if (method == "fitness")                                 # fitness-based probabilities
  {
    fitness_weight <- (fitness - min(fitness) + par[1]) ^  # weight according 
                      par[2]                               # to the fitness
    probs <- fitness_weight / sum(fitness_weight)          # assign probabilities
  }                                                        # proportional to weights
  
  # Select the parents based on probabilities
  
  parents_ind <- rep(NA, parents.n)                        # vector to store
                                                           # parent indices
  
  if (method %in% c("constant", "rank", "fitness"))
  {
    parents_ind <- sample(ind_pop,                         # indexes of parents
                          size = parents.n,            
                          replace = TRUE, 
                          prob = probs)
  }
  
  # Select parents following some other criteria
  
  if (method == "tournament")                              # select parents 
  {                                                        # via the tournament
    pop_ind <- 1:pop.n
    parents_ind <- mating_tournament(                      # Rcpp version for
      pop_ind = pop_ind, candidates_n = par[1],            # speed improvement
      fitness = fitness, parents_n = parents.n)                                 
    # for(i in 1:parents.n)
    # {
    #   candidates <- sample(pop_ind,                      # randomly pick
    #                        size = par[1],                # some candidates
    #                        replace = FALSE)
    #   parents_ind[i] <- candidates[                      # select the fittest
    #     which.max(fitness[candidates])]                  # of all candidates
    # }
  }
    # ---
    # Comment to tournament method:
    # It is possible to assign the probabilities
    # but it requires combinatorial formulas to be
    # used that may cause numeric instability
    # ---
  
  # Prevent self-mating
  
  odd_ind <- NULL
  
  if (!self)
  {
    odd_ind <- seq(from = 1, to = parents.n, by = 2)
    if (method %in% c("constant", "rank", "fitness"))
    {
      for (i in odd_ind)
      {
        if (parents_ind[i] == parents_ind[i + 1])
        {
          parents_ind[i] <- sample(ind_pop[-parents_ind[i]],
                                   size = 1,            
                                   replace = TRUE, 
                                   prob = probs[-parents_ind[i]] / 
                                          sum(probs[-parents_ind[i]]))
        }
      }
    }
  }
  
  # Save and return the results of selection
  
  parents <- population[parents_ind, , drop = FALSE]       # parents and
  parents_fitness <- fitness[parents_ind]                  # their fitness
  
  return_val <- list(parents = parents,                    # output in the form
                     fitness = parents_fitness,            # of a list
                     ind = parents_ind)            
  
  return(return_val)                                       # return the parents
                                                           # along with their 
                                                           # fitness
}

# Assign default parameters for
# a mating algorithm depending
# on the "method"
gena.mating.validate <- function(method, par, parents.n)
{
  # Validate the "method"
  
  methods <- c("constant", "rank", "fitness",              # the list of all
               "tournament")                               # available methods
                                                           
  if (!(method %in% methods))
  {
    stop(paste0("Incorrect mating.method argument. ",      # if the user has provided
                "It should be one of: ",                   # incorrect argument
                paste(methods, collapse = ", "),
                "\n"))
  }
  
  # Assign default parameters
  
  if (method == "rank")
  {
    if (!is.null(par))
    {
      if ((length(par) != 1) | (!is.numeric(par)))
      {
        stop(paste0("Incorrect mating.par argument. Please, ensure that ",
                    "(length(mating.par) == 1) and is.numeric(mating.par)",
                    "\n"))
      }
    } else {
      par <- 0.5
    }
  }
  
  if (method == "fitness")
  {
    if (!is.null(par))
    {
      if ((length(par) != 2) | (par[1] < 0) | (!is.numeric(par)))
      {
        stop(paste0("Incorrect mating.par argument. Please, ensure that ",
                    "(length(mating.par) == 2), (mating.par[1] > 0) ",
                    "and is.numeric(mating.par)",
                    "\n"))
      }
    } else {
      par <- c(10, 0.5)
    }
  }
  
  if (method == "tournament")
  {
    if (!is.null(par))
    {
      if ((length(par) != 1) | (par < 1) | (par > parents.n) | 
         (!is.numeric(par)))
      {
        stop(paste0("Incorrect mating.par argument. Please, ensure that ",
                    "(length(mating.par) == 1), (mating.par >= 1)",
                    "(mating.par < (pop.n - n_elite)), and ",
                    "is.numeric(mating.par)",
                    "\n"))
      }
    } else {
      par <- min(5, ceiling(parents.n * 0.1))
    }
  }
  
  return(par)
}

#' Mutation
#' @description Mutation method (algorithm) to be used in the
#' genetic algorithm.
#' @param children numeric matrix whose rows are children, i.e., vectors of 
#' parameter values.
#' @param lower lower bound of the search space.
#' @param upper upper bound of the search space.
#' @param prob probability of mutation for a child.
#' @param prob.genes numeric vector or numeric value representing the
#' probability of mutation of a child's gene. See 'Details'.
#' @param method mutation method to be used for transforming genes of children.
#' @param par additional parameters to be passed depending on the \code{method}.
#' @param iter iteration number of the genetic algorithm.
#' 
#' @details Denote \code{children} by \eqn{C^{child}} whose \code{i}-th row 
#' \code{children[i, ]} is a chromosome \eqn{c_{i}^{child}}, i.e., the vector of 
#' parameter values of the function being optimized \eqn{f(.)} that is
#' provided via the \code{fn} argument of \code{\link[gena]{gena}}.
#' The elements of chromosome \eqn{c_{ij}^{child}} are genes 
#' representing parameters values.
#' 
#' Mutation algorithm determines random transformation of children's genes. 
#' Each child may be selected for mutation with probability \code{prob}.
#' If \eqn{i}-th child is selected for mutation and \code{prob.genes} is a 
#' vector then \eqn{j}-th gene of this child
#' is transformed with probability \code{prob.genes[j]}. If \code{prob.genes}
#' is a constant then this probability is the same for all genes.
#' 
#' Argument \code{method} determines the particular mutation algorithm to 
#' be applied. Denote by \eqn{\tau} the vector of parameters used by the 
#' algorithm. Note that \eqn{\tau} corresponds to \code{par}.
#' Also let us denote by \eqn{c_{ij}^{mutant}} the value of 
#' gene \eqn{c_{ij}^{child}} after mutation.
#' 
#' If \code{method = "constant"}, then \eqn{c_{ij}^{mutant}}
#' is a uniform random variable between \code{lower[j]} and \code{upper[j]}.
#' 
#' If \code{method = "normal"}, then \eqn{c_{ij}^{mutant}}
#' equals the sum of \eqn{c_{ij}^{child}} and a normal random variable
#' with zero mean and standard deviation \code{par[j]}. 
#' By default, \code{par} is a vector of ones of length \code{ncol(children)},
#' so \code{par[j] = 1} for all \code{j}.
#' 
#' If \code{method = "percent"}, then \eqn{c_{ij}^{mutant}} is generated 
#' from \eqn{c_{ij}^{child}}  by equiprobably increasing or decreasing it 
#' by \eqn{q} percent,
#' where \eqn{q} is a uniform random variable between \eqn{0} and \code{par[j]}. 
#' Note that \code{par} may also be a constant; then all
#' genes have the same maximum possible percentage change.
#' By default, \code{par = 20}.
#' 
#' For more information on mutation algorithms,
#' please see Patil and Bhende (2014).
#' 
#' @return The function returns a matrix whose rows are children
#' (after mutation has been applied to some of them).
#' 
#' @references S. Patil, M. Bhende. (2014). 
#' Comparison and Analysis of Different Mutation Strategies to improve the 
#' Performance of Genetic Algorithm.
#' \emph{International Journal of Computer Science and 
#' Information Technologies}, 5 (3), 4669-4673.
#' 
#' @examples 
#' # Randomly initialize some children
#' set.seed(123)
#' children.n <- 10
#' children <- gena.population(pop.n = children.n,
#'                             lower = c(-5, -5), 
#'                             upper = c(5, 5))
#'                            
#' # Perform the mutation
#' mutants <- gena.mutation(children = children,
#'                          prob = 0.6,
#'                          prob.genes = c(0.7, 0.8),
#'                          par = 30,
#'                          method = "percent")
#' print(mutants)

# Mutation operator
gena.mutation <- function(children,                        # children
                          lower,                           # lower bound of search space
                          upper,                           # upper bound of search space
                          prob = 0.2,                      # mutation probability for children
                          prob.genes = 1 / nrow(children), # mutation probabilities for genes
                          method = "constant",             # mutation type
                          par = 1,                         # mutation parameters
                          iter = NULL)                     # iteration of the
                                                           # genetic algorithm
{
  # ---
  # The algorithm brief description:
  # 1. Randomly pick the children for mutation
  # 2. Perform the mutation for each gene of each child
  # ---
  
  # Prepare some variables
  
  children.n <- nrow(children)                             # number of children
  genes.n <- ncol(children)                                # number of genes
  
  # Select children for mutation

  random_values <- runif(children.n, 0, 1)                 # values to control for
                                                           # whether mutation should
                                                           # take place for a given
                                                           # child
  mutation_ind <- which(random_values <= prob)             # select the children
                                                           # to mutate
  
  for (i in mutation_ind)
  {
    random_values_genes <- runif(genes.n, 0, 1)
    genes_ind <- which(prob.genes <= random_values_genes)
    
    if (method == "constant")
    {
      for (j in genes_ind)
      {
        children[i, j] <- runif(1, lower[j], upper[j])
      }
    }
    
    if (method == "normal")
    {
      for (j in genes_ind)
      {
        children[i, j] <- children[i, j] + rnorm(1, mean = 0, sd = par[j])
        # children[i, j] <- min(max(children[i, j], lower[j]), upper[j])
      }
    }
    
    if (method == "percent")
    {
      par_adj <- par / 100
      children_adj <- abs(children[i, ]) * par_adj
      mutation_lower <- children[i, ] - children_adj
      mutation_upper <- children[i, ] + children_adj
      for (j in genes_ind)
      {
        children[i, j] <- runif(1, mutation_lower[j], mutation_upper[j])
        # children[i, j] <- min(max(children[i, j], lower[j]), upper[j])
      }
    }
  }
  
  return(children)                                         # return the children
}

# Assign default parameters for
# mutation algorithm depending
# on the "method"
gena.mutation.validate <- function(method, par, genes.n)
{
  # Validate the "method"
  
  methods <- c("constant", "normal", "percent")
  
  if (!(method %in% methods))
  {
    stop(paste0("Incorrect mutation.method argument. ",    # if the user has provided
                "It should be one of: ",                   # incorrect argument
                paste(methods, collapse = ", "),
                "\n"))
  }
  
  # Assign default parameters
  
  if (method == "normal")
  {
    if (is.null(par))
    {
      par <- 1
    } else {
      if (any((par <= 0)))
      {
        stop("par should be a vector with positive values")
      }
    }
    
    if (length(par) != genes.n)
    {
      if (length(par) == 1)
      {
        par <- rep(par, genes.n)
      } else {
        stop("length(par) should be equal to length(lower)")
      }
    }
  }
  
  if (method == "percent")
  {
    if (is.null(par))
    {
      par <- rep(20, genes.n)
    } else {
      if (length(par) == 1)
      {
        par <- rep(par, genes.n)
      } else {
        if (length(par) > genes.n)
        {
          stop(paste0("Please, ensure that 'par' has the same size as the ",
                      "number of genes, i.e., 'length(par)==ncol(lower)'"))
        }
      }
      
      if (any((par < 0)))
      {
        stop("par should not have negative values")
      }
    }
    
    if (length(par) != genes.n)
    {
      if (length(par) == 1)
      {
        par <- rep(par, genes.n)
      } else {
        stop("length(par) should be equal to length(lower)")
      }
    }
  }
  
  return(par)
}

#' Crossover
#' @description Crossover method (algorithm) to be used in the
#' genetic algorithm.
#' @param parents numeric matrix whose rows are parents, i.e., vectors of 
#' parameters values.
#' @param fitness numeric vector whose \code{i}-th element is the value of 
#' \code{fn} at point \code{population[i, ]}.
#' @param prob probability of crossover.
#' @param method crossover method to be used for making children.
#' @param par additional parameters to be passed depending on the \code{method}.
#' @param iter iteration number of the genetic algorithm.
#' @details Denote \code{parents} by \eqn{C^{parent}} whose \code{i}-th row 
#' \code{parents[i, ]} is a chromosome \eqn{c_{i}^{parent}}, i.e., the vector of 
#' parameter values of the function being optimized \eqn{f(.)} that is
#' provided via the \code{fn} argument of \code{\link[gena]{gena}}.
#' The elements of chromosome \eqn{c_{ij}^{parent}} are genes 
#' representing parameters values.
#' 
#' Crossover algorithm determines the way parents produce children. 
#' During crossover, each of the randomly selected pairs of parents 
#' \eqn{c_{i}^{parent}}, \eqn{c_{i + 1}^{parent}}
#' produces two children 
#' \eqn{c_{i}^{child}}, \eqn{c_{i + 1}^{child}}, 
#' where \eqn{i} is odd. Each pair of parents is selected with 
#' probability \code{prob}. If a pair of parents has not been selected
#' for crossover, then corresponding children and parents coincide, i.e., 
#' \eqn{c_{i}^{child}=c_{i}^{parent}} and
#' \eqn{c_{i+1}^{child}=c_{i+1}^{parent}}.
#' 
#' Argument \code{method} determines particular crossover algorithm to 
#' be applied. Denote by \eqn{\tau} the vector of parameters used by the 
#' algorithm. Note that \eqn{\tau} corresponds to \code{par}.
#' 
#' If \code{method = "split"} then each gene of the first child will
#' be equiprobably picked from the first or from the second parent. So 
#' \eqn{c_{ij}^{child}} may be equal to \eqn{c_{ij}^{parent}}
#' or \eqn{c_{(i+1)j}^{parent}} with equal probability. The second
#' child is the reversal of the first one in the sense that if the first child 
#' gets a particular gene of the first (second) parent, then the second child 
#' gets this gene from the second (first) parent, i.e., if
#' \eqn{c_{ij}^{child}=c_{ij}^{parent}} then 
#' \eqn{c_{(i+1)j}^{child}=c_{(i+1)j}^{parent}}; if 
#' \eqn{c_{ij}^{child}=c_{(i+1)j}^{parent}} then 
#' \eqn{c_{(i+1)j}^{child}=c_{ij}^{parent}}.
#' 
#' If \code{method = "arithmetic"} then:
#' \deqn{c_{i}^{child}=\tau_{1}c_{i}^{parent}+
#' \left(1-\tau_{1}\right)c_{i+1}^{parent}}
#' \deqn{c_{i+1}^{child}=\left(1-\tau_{1}\right)c_{i}^{parent}+
#' \tau_{1}c_{i+1}^{parent}}
#' where \eqn{\tau_{1}} is \code{par[1]}. By default \code{par[1] = 0.5}.
#' 
#' If \code{method = "local"} then the procedure is the same as 
#' for "arithmetic" method but \eqn{\tau_{1}} is a uniform random
#' value between 0 and 1.
#' 
#' If \code{method = "flat"} then \eqn{c_{ij}^{child}} is a uniform
#' random number between \eqn{c_{ij}^{parent}} and 
#' \eqn{c_{(i+1)j}^{parent}}. 
#' Similarly for the second child \eqn{c_{(i+1)j}^{child}}.
#' 
#' For more information on crossover algorithms,
#' please see Kora and Yadlapalli (2017).
#' 
#' @return The function returns a matrix whose rows are children.
#' 
#' @references P. Kora, P. Yadlapalli. (2017). 
#' Crossover Operators in Genetic Algorithms: A Review.
#' \emph{International Journal of Computer Applications}, 162 (10), 34-36,
#' <doi:10.5120/ijca2017913370>.
#' @examples 
#' # Randomly initialize the parents
#' set.seed(123)
#' parents.n <- 10
#' parents <- gena.population(pop.n = parents.n,
#'                            lower = c(-5, -5), 
#'                            upper = c(5, 5))
#'                            
#' # Perform the crossover
#' children <- gena.crossover(parents = parents,
#'                            prob = 0.6,
#'                            method = "local")
#' print(children)

# Crossover the parents
gena.crossover <- function(parents,                        # the parents
                           fitness = NULL,                 # fitness values of chromosomes
                           prob = 0.8,                     # crossover probability
                           method = "local",               # crossover type
                           par = NULL,                     # crossover parameters
                           iter = NULL)                    # iteration of the
                                                           # genetic algorithm
{
  # ---
  # The algorithm brief description:
  # 1. Randomly pick the parents for the crossover
  # 2. Perform the crossover for each of randomly 
  #    selected pair of parents
  # ---
  
  # Prepare some variables
  
  parents.n <- nrow(parents)                               # number of parents
  genes.n <- ncol(parents)                                 # number of genes
  
  children <- parents                                      # matrix to store children
                                                           # initially contains
                                                           # the parents

  # Perform the crossover for each chromosome
  
  random_values <- runif(parents.n / 2, 0, 1)              # values to control for
                                                           # whether crossover should
                                                           # take place for a given
                                                           # pair of chromosomes
  crossover_ind <- which(random_values <= prob) * 2        # select the pairs
                                                           # for a crossover
  
  for (i in crossover_ind)
  {
    ind <- c(i - 1, i)                                     # indexes of the first 
                                                           # and the second parents 
    
    if (method == "split")
    {
      split_ind <- which(rbinom(genes.n, 1, 0.5) == 1)     # indexes to get from
                                                           # the other parent
      children[ind[1], split_ind] <- parents[ind[2],       
                                             split_ind] 
      children[ind[2], split_ind] <- parents[ind[1], 
                                             split_ind]
    }
    
    if (method == "arithmetic")
    {
      children[ind[1], ] <- parents[ind[1], ] * par[1] +      # mix the parents with
        parents[ind[2], ] * (1 - par[1])                      # a given weight
      children[ind[2], ] <- parents[ind[2], ] * par[1] +
        parents[ind[1], ] * (1 - par[1])
    }
    
    if (method == "local")                                    # local crossover
    {
      weight <- runif(n = genes.n, min = 0, max = 1)          # random weight
      children[ind[1], ] <- parents[ind[1], ] * weight +      # mix the parents with
                            parents[ind[2], ] * (1 - weight)  # a random weight
      children[ind[2], ] <- parents[ind[2], ] * weight +
                            parents[ind[1], ] * (1 - weight)
    }
    
    if (method == "flat")                                          # flat crossover
    {
      for (j in 1:genes.n)                                         # for each gene
      {                                                            # and for both
        for (t in 1:2)                                             # children
        {
          children[ind[t], j] <- runif(n = 1,                      # randomly pick    
                                       min(parents[ind[1], j],     # the value between   
                                           parents[ind[2], j]),    # the genes of
                                       max(parents[ind[1], j],     # the parents
                                           parents[ind[2], j]))
        }
      }
    }
  }

  return(children)                                         # return the children
}

# Assign default parameters for
# crossover algorithm depending
# on the "method"
gena.crossover.validate <- function(method, par)
{
  # Validate the "method"
  
  methods <- c("split", "arithmetic", "local", "flat")     # the list of all
                                                           # available methods
  
  if (!(method %in% methods))
  {
    stop(paste0("Incorrect crossover.method argument. ",   # if the user has provided
                "It should be one of: ",                   # incorrect argument
                paste(methods, collapse = ", "),
                "\n"))
  }
  
  # Assign default parameters
  
  if (method == "arithmetic")
  {
    if (!is.null(par))
    {
      if ((length(par) != 1) | (!is.numeric(par)))
      {
        stop(paste0("Incorrect crossover.par argument. Please, ensure that ",
                    "(length(crossover.par) == 1) and ",
                    "is.numeric(crossover.par)",
                    "\n"))
      }
    } else {
      par <- 0.5
    }
  }
  
  return(par)
}

#' Hybridization
#' @description Hybridization method (algorithm) to be used in the
#' genetic algorithm.
#' @param population numeric matrix whose rows are chromosomes, i.e., vectors of 
#' parameter values.
#' @param fitness numeric vector whose \code{i}-th element is the value of 
#' \code{fn} at point \code{population[i, ]}.
#' @param hybrid.n positive integer representing the number of hybrids.
#' @param method hybridization method to improve chromosomes via local search.
#' @param par additional parameters to be passed depending on the \code{method}.
#' @param opt.par parameters of the local optimization function
#' to be used for hybridization algorithm (including \code{fn} and \code{gr}).
#' @param info logical; if \code{TRUE} then some optimization-related 
#' information will be printed each iteration.
#' @param iter iteration number of the genetic algorithm.
#' @param ... additional parameters to be passed to 
#' \code{fn} and \code{gr} functions.
#' @details This function uses \code{\link[gena]{gena.mating}} function to 
#' select hybrids. Therefore \code{method} and \code{par} arguments will
#' be passed to this function. If some chromosomes selected to become hybrids
#' are duplicated, then these duplicates will not be subject to local
#' optimization, i.e., the number of hybrids will be decreased by the number
#' of duplicates (actual number of hybrids during some iterations may be 
#' lower than \code{hybrid.n}).
#' 
#' Currently \code{\link[stats]{optim}} is the only available local
#' optimizer. Therefore \code{opt.par} is a list containing parameters
#' that should be passed to \code{\link[stats]{optim}}.
#' 
#' For more information on hybridization, please see El-Mihoub et al. (2006).
#' 
#' @return The function returns a list with the following elements:
#' \itemize{
#' \item \code{population} - matrix whose rows are chromosomes including hybrids.
#' \item \code{fitness} - vector whose i-th element is the fitness of the
#' i-th chromosome.
#' \item \code{hybrids.ind} - vector of indexes of chromosomes selected for
#' hybridization.
#' \item \code{counts} - a two-element integer vector giving the number of
#' calls to \code{fn} and \code{gr} respectively.
#' }
#' 
#' @references T. El-Mihoub, A. Hopgood, L. Nolle, B. Alan (2006). 
#' Hybrid Genetic Algorithms: A Review.
#' \emph{Engineering Letters}, 13 (3), 124-137.
#' 
#' @examples 
#' # Consider the following fitness function
#' fn <- function(x)
#' {
#'   val <- x[1] * x[2] - x[1] ^ 2 - x[2] ^ 2
#' }
#' 
#' # Also let's provide its gradient (optional)
#' gr <- function(x)
#' {
#'   val <- c(x[2] - 2 * x[1],
#'            x[1] - 2 * x[2])
#' }
#' 
#' # Randomly initialize the population
#' set.seed(123)
#' n_population <- 10
#' population <- gena.population(pop.n = n_population,
#'                               lower = c(-5, -5), 
#'                               upper = c(5, 5))
#'
#' # Calculate fitness of each chromosome
#' fitness <- rep(NA, n_population)
#' for(i in 1:n_population)
#' {
#'   fitness[i] <- fn(population[i, ])
#' }
#' 
#' # Perform hybridization
#' hybrids <- gena.hybrid(population = population,
#'                        fitness = fitness,
#'                        opt.par = list(fn = fn,
#'                                       gr = gr,
#'                                       method = "BFGS",
#'                                       control = list(fnscale = -1,
#'                                                      abstol = 1e-10,
#'                                                      reltol = 1e-10,
#'                                                      maxit = 1000)),
#'                        hybrid.n = 2,
#'                        method = "rank",
#'                        par = 0.8)
#' print(hybrids)
#' 

# Make hybrids
gena.hybrid <- function(population,
                        fitness,
                        hybrid.n = 1,
                        method,
                        par,
                        opt.par,
                        info = FALSE,
                        iter = NULL,
                        ...)          
{      
  # ---
  # The algorithm brief description:
  # 1. Select the chromosomes to become the hybrids
  # 2. Perform local optimization on chromosomes
  #    to make them the hybrids
  # ---
  
  # Prepare some values
  
  n_pop <- nrow(population)                                # number of chromosomes
  genes.n <- ncol(population)                              # number of genes
  
  dots <- list(...)                                        # fn specific parameters
  all.args <- append(opt.par, dots)                        # all parameters to be
                                                           # passed to optim
  
  # Maximization or minimization
  is.max <- FALSE
  if (hasName(x = opt.par, name = "control"))
  {
    if (hasName(x = opt.par$control, name = "fnscale"))
    {
      is.max <- opt.par$control$fnscale < 0
    }
  }

  # Get the name of the first argument of the function
  
  fn.par <- names(formals(all.args$fn))[1]
  
  # To select hybrids use the same function
  # as for the parents
  
  hybrids_list <- gena.mating(population = population,     # select the chromosomes
                              fitness = fitness,           # to become hybrids
                              parents.n = hybrid.n,
                              method = method,
                              par = par,
                              iter = iter,
                              self = TRUE)
  hybrids.ind <- unique(hybrids_list$ind)                  # indexes of chromosomes that
                                                           # will become hybrids
  # Perform the optimization
  
  counter <- 1
  counts <- c(0, 0)
  for (i in hybrids.ind)
  {
    hybrid_message <- paste0("hybrid ", counter, " fitness: before = ",
                             fitness[i],
                             ", after = ")
    
    all.args$par <- population[i, ]
    opt.result <- NULL
    tryCatch(
    {
      opt.result <- do.call(optim, args = all.args)
      if (!is.null(opt.result[["counts"]]))
      {
        optim.counts <- opt.result$counts
        optim.counts[is.na(optim.counts)] <- 0
        counts <- counts + optim.counts
      }
      if (!hasName(x = opt.result, name = "value"))
      {
        opt.result$value <- -Inf
      }
      if ((is.max  & (fitness[i] < opt.result$value)) |
          (!is.max & (fitness[i] > opt.result$value)))
      {
        population[i, ] <- opt.result$par
        fitness[i] <- opt.result$value
        dots$par <- opt.result$par
      }
    })
    hybrid_message <- paste0(hybrid_message, fitness[i], "\n")
    if (info)
    {
      cat(hybrid_message)
    }
    counter <- counter + 1
  }
  
  return_list <- list(population = population,
                      fitness = fitness,
                      hybrids.ind = hybrids.ind,
                      counts = counts)
  
  return(return_list)
  
  # ---
  # Ideas for the future development
  # ---
  # 1. Add argument type - string representing a type of the selection mechanism.
  #   If \code{hybrid.type = "Lamarck"} (default) then hybridization changes
  #   both genes and fitness of the corresponding chromosome.
  #   If \code{hybrid.type = "Baldwin"} then only the fitness changes.
  #   If this argument is double between \code{0} and \code{1} then this 
  #   number represents probability of the Lamarck search.
  # ---
}

# References
# Best BHGA: https://www.hindawi.com/journals/jam/2013/103591/

#' Population
#' @description Initialize the population of chromosomes.
#' @param pop.n positive integer representing the number of chromosomes 
#' in population.
#' @param lower numeric vector whose i-th element determines the minimum
#' possible value for i-th gene.
#' @param upper numeric vector whose i-th element determines the maximum
#' possible value for i-th gene.
#' @param pop.initial numeric matrix whose rows are initial chromosomes
#' suggested by user.
#' @param method string representing the initialization method to be used.
#' For a list of possible values see Details.
#' @details If \code{method = "uniform"}, then the i-th gene of each 
#' chromosome is randomly (uniformly) chosen between \code{lower[i]} and 
#' \code{upper[i]}. If \code{method = "normal"}, then the i-th gene is 
#' generated from a truncated normal distribution with the mean 
#' \code{(upper[i] + lower[i]) / 2} and standard deviation 
#' \code{(upper[i] - lower[i]) / 6}, where \code{lower[i]} and \code{upper[i]} 
#' are the lower and upper truncation bounds, respectively.
#' If \code{method = "hypersphere"}, then the population is simulated uniformly
#' from the hypersphere with center \code{upper - lower} and radius
#' \code{sqrt(sum((upper - lower) ^ 2))} via the 
#' \code{\link[gena]{rhypersphere}} function, setting \code{type = "inside"}.
#' @return This function returns a matrix whose rows are chromosomes.
#' @references B. Kazimipour, X. Li, A. Qin (2014). 
#' A review of population initialization techniques for evolutionary algorithms.
#' \emph{2014 IEEE Congress on Evolutionary Computation}, 2585-2592,
#' <doi:10.1109/CEC.2014.6900618>.
#' @examples
#' set.seed(123)
#' gena.population(pop.n = 10,
#'                 lower = c(-1, -2, -3),
#'                 upper = c(1, 0, -1),
#'                 pop.initial = rbind(c(0, -1, -2),
#'                                     c(0.1, -1.2, -2.3)),
#'                 method = "normal")
                
gena.population <- function(pop.n,                         # the size of the population
                            lower,                         # lower bounds for the parameters
                            upper,                         # upper bounds for the parameters
                            pop.initial = NULL,            # initial chromosomes suggested by user
                            method = "uniform")            # population initialization algorithm
{
  # Validation
  
  methods <- c("uniform", "normal", "hypersphere")         # the list of all
                                                           # available methods
    
  if (!(method %in% methods))
  {
    stop(paste0("Incorrect population.method argument. ",  # if the user has provided
                "It should be one of: ",                   # incorrect argument
                paste(methods, collapse = ", "),
                "\n"))
  }

  # Prepare some values
  
  genes.n <- length(lower)                                 # the number of parameters
  pop.n.0 <- NULL                                          # the number of population members
                                                           # suggested by the user
  pop.n.1 <- pop.n                                         # the number of population members
                                                           # to be created    
  # Deal with chromosomes proposed
  # by the user
  
  if (!is.null(pop.initial))                               # if the user has provided any
  {                                                        # initial chromosomes
    pop.n.0 <- nrow(pop.initial)                           # the number of chromosomes 
                                                           # provided by the user
    if (pop.n.0 == pop.n)                                  # if the user has provided
    {                                                      # the entire population
      return(pop.initial)                                  # simply return the user
    }                                                      # provided population
    
    pop.n.1 <- pop.n - pop.n.0                             # the number of chromosomes
                                                           # to be created
  }

  # Initialize the population
  
  population <- matrix(NA,                                 # population matrix
                       nrow = pop.n.1, 
                       ncol = genes.n)   
  
  if (method == "uniform")                                 # create the chromosomes using
  {                                                        # the uniform distribution
    for (i in 1:genes.n)
    {
      population[, i] <- runif(pop.n.1, 
                               lower[i], upper[i])
    }
  }
  
  if (method == "normal")                                  # create the chromosomes using
  {                                                        # normal distribution
    mu <- (upper + lower) / 2  
    sigma <- (upper - mu) / 3
    p_lower <- pnorm(lower)
    p_upper <- pnorm(upper)
    p_diff <- p_upper - p_lower
    for (i in 1:genes.n)
    {
      u <- runif(pop.n.1)
      population[, i] <- qnorm(p_lower[i] + 
                               u * p_diff[i]) *
                         sigma[i] + mu[i]
    }
  }
  
  if (method == "hypersphere")
  {
    population <- rhypersphere(n = pop.n.1, dim = genes.n,
                               radius = sqrt(sum((upper - lower) ^ 2)),
                               center = upper - lower,
                               type = "inside")
  }
  
  population <- rbind(pop.initial, population)             # combine the chromosomes provided
                                                           # by the user and the chromosomes
                                                           # that were initialized
                                                         
  return(population)                                       # return the population of chromosomes
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @export
mating_tournament <- function(pop_ind, candidates_n, fitness, parents_n) {
    .Call(`_gena_mating_tournament`, pop_ind, candidates_n, fitness, parents_n)
}

#' Velocity
#' @description Calculates (updates) velocities of the particles.
#' @param population numeric matrix whose rows are particles, i.e., vectors of 
#' parameters values.
#' @param method string representing method to be used for velocities 
#' calculation. See 'Details' for additional information.
#' @param par additional parameters to be passed depending on the \code{method}.
#' @param velocity matrix whose i-th row is a velocity of the i-th particle. 
#' @param best.pn numeric matrix whose i-th row is a best personal position 
#' known by the i-th particle.
#' @param best.nh numeric matrix whose i-th row is a best personal position 
#' in a neighbourhood of the i-th particle.
#' @param best.pn.fitness numeric vector whose i-th element is the value of
#' the fitness function at point \code{best.pn[i, ]}.
#' @param best.nh.fitness numeric vector whose i-th element is the value of
#' the fitness function at point \code{best.nh[i, ]}.
#' @param iter iteration number of the genetic algorithm.
#' @details If \code{method = "classic"}, then the classical velocity formula
#' is used:
#' \deqn{v_{i,j,(t+1)}=w\times v_{i,j,t} + 
#'                      c_{1}\times u_{1,i,j} \times b^{pn}_{i,j,t} + 
#'                      c_{2}\times u_{2,i,j} \times b^{nh}_{i,j,t}}
#'                      
#'  where \eqn{v_{i, j, t}} is the velocity of the \eqn{i}-th particle 
#'  with respect to the \eqn{j}-th component at time \eqn{t}. 
#'  The random variables \eqn{u_{1,i,j}} and \eqn{u_{2,i,j}} are i.i.d. with 
#'  respect to all indices and follow the standard uniform distribution 
#'  \eqn{U(0, 1)}. The variable \eqn{b^{pn}_{i,j,t}} is the \eqn{j}-th 
#'  component of the best known personal position of the particle up to time 
#'  period \eqn{t}. Similarly, \eqn{b^{nh}_{i,j,t}} is the \eqn{j}-th component 
#'  of the best known position in the neighbourhood of the \eqn{i}-th particle.
#'  The hyperparameters \eqn{w}, \eqn{c_{1}}, and \eqn{c_{2}} may be provided
#'  via the \code{par} argument as a list with elements \code{par$w}, 
#'  \code{par$c1}, and \code{par$c2}, respectively. 
#'  
#'  If \code{method = "hypersphere"}, then rotation-invariant formula from
#'  sections 3.4.2 and 3.4.3 of M. Clerc (2012) is used, with arguments
#'  identical to the classical method. To simulate a random variate from
#'  the hypersphere, the function \code{\link[gena]{rhypersphere}} is used,
#'  setting \code{type = "non-uniform"}.
#'  
#'  In accordance with M. Clerc (2012), the 
#'  default values are \code{par$w = 1/(2 * log(2))},
#'  \code{par$c1 = 0.5 + log(2)}, and \code{par$c2 = 0.5 + log(2)}.
#'  
#' @return This function returns a matrix whose i-th row represents the 
#' updated velocity of the i-th particle.
#'  
#' @references Maurice Clerc (2012). 
#' Standard Particle Swarm Optimisation.
#' \emph{HAL archive}.
#'              
pso.velocity <- function(population,
                         method = "hypersphere",
                         par = list(w = 1 / (2 * log(2)), 
                                    c1 = 0.5 + log(2),
                                    c2 = 0.5 + log(2)),
                         velocity,
                         best.pn,
                         best.nh,
                         best.pn.fitness,
                         best.nh.fitness,
                         iter = 1)              
{
  # Dimensional variables
  
  pop.n <- nrow(population)
  genes.n <- ncol(population)
  total.n <- genes.n * pop.n
  
  # Determine best particles in a neighbourhood
  
  is.best <- best.pn.fitness == best.nh.fitness
  
  # Matrix of updated velocities
  
  velocity.new <- NULL
  
  # Update velocities according to method
  
  if (method == "classic")
  {
    # Simulate from uniform distribution
    
    unif.1 <- matrix(runif(total.n), nrow = pop.n, ncol = genes.n)
    unif.2 <- matrix(runif(total.n), nrow = pop.n, ncol = genes.n)
    unif.2[is.best] <- 0
    
    # Calculate the velocity
    
    velocity.new <- par$w * velocity +
                    par$c1 * unif.1 * (best.pn - population) +
                    par$c2 * unif.2 * (best.nh - population)
  }
  
  if (method == "hypersphere")
  {
    # Simulate uniform random variates from the hypersphere
    
    adj <- matrix(NA, nrow = pop.n, ncol = genes.n)
    adj[is.best] <- (par$c1 / 2) * (best.pn[is.best, ] - population[is.best, ])
    adj[!is.best] <- (par$c1 / 3) * (best.pn[!is.best, ] - population[!is.best, ]) +
                     (par$c2 / 3) * (best.nh[!is.best, ] - population[!is.best, ])
    center <- population + adj
    radius <- sqrt(rowSums(adj ^ 2))
    x <- rhypersphere(n = pop.n, 
                      dim = genes.n, 
                      radius = radius,
                      center = center,
                      type = "non-uniform")

    # Calculate the velocity
    
    velocity.new <- par$w * velocity + x - population
  }
  
  return(velocity.new)
}

# Assign default parameters for
# velocity algorithm depending
# on the "method"
pso.velocity.validate <- function(method, par)
{
  # Validate the "method"
  
  methods <- c("classic", "hypersphere")                   # the list of all
                                                           # available methods
  if (!(method %in% methods))
  {
    stop(paste0("Incorrect velocity.method argument. ",    # if the user has provided
                "It should be one of: ",                   # incorrect argument
                paste(methods, collapse = ", "),
                "\n"))
  }
  
  if (method %in% c("classic", "hypersphere"))
  {
    if (is.null(par))
    {
      par <- list(w = 1 / (2 * log(2)), 
                  c1 = 0.5 + log(2),
                  c2 = 0.5 + log(2))
    }
    else
    {
      
      if (!is.list(par))
      {
        par.vec <- par
        par.vec.n <- length(par.vec)
        if ((par.vec.n != 2) & (par.vec.n != 3))
        {
          stop("If 'velocity.par' is a vector, then it should contain 2 or 3 elements.")
        }
        par <- list(w = par.vec[1], 
                    c1 = par.vec[2])
        if (length(par.vec) == 2)
        {
          par$c2 <- par$c1
        }
      }
      else
      {
        if (!all(c("w", "c1", "c2") %in% names(par)))
        {
          stop("Argument 'velocity.par' should be a list containing 'w', 'c1', and 'c2'.")
        }
      }
    }
  }
  
  return(par)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/random.R
\name{rhypersphere}
\alias{rhypersphere}
\title{Hypersphere}
\usage{
rhypersphere(n, dim = 2, radius = 1, center = rep(0, dim), type = "boundary")
}
\arguments{
\item{n}{number of observations to simulate.}

\item{dim}{dimension of the hypersphere.}

\item{radius}{radius of the hypersphere.}

\item{center}{center of  the hypersphere.}

\item{type}{character; if \code{"boundary"} (default) then random variates 
are simulated from the hypersphere. If \code{"inside"}, the random
variates are points lying inside the hypersphere. If \code{"non-uniform"}, 
then random variates are non-uniform and are simulated from the inner part
of the hypersphere by making the radius a uniform random variable
between \code{0} and \code{radius}.}
}
\value{
The function returns a matrix whose rows are random variates 
(when \code{n > 1}) or a vector (when \code{n = 1}).
}
\description{
Simulates uniform random variates from the hypersphere.
}
\examples{
set.seed(123)
# Get 5 random uniform variates from 3D hypersphere
# of radius 10 centered at (2, 3, 1)
rhypersphere(n = 5, dim = 3, radius = 10, center = c(2, 3, 1))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pso.R
\name{summary.pso}
\alias{summary.pso}
\title{Summarizing pso Fits}
\usage{
\method{summary}{pso}(object, ...)
}
\arguments{
\item{object}{Object of class "pso"}

\item{...}{further arguments (currently ignored)}
}
\value{
This function returns the same list as \code{\link[gena]{pso}} 
function changing its class to "summary.pso".
}
\description{
Summarizing pso Fits
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/neighbourhood.R
\name{pso.nh}
\alias{pso.nh}
\title{Neighbourhood}
\usage{
pso.nh(pop.n = 40, method = "ring", par = 3, iter = 1)
}
\arguments{
\item{pop.n}{integer representing the size of the population.}

\item{method}{string representing the topology to be used for construction
of the neighbourhood. See 'Details' for additional information.}

\item{par}{additional parameters to be passed depending on the \code{method}.}

\item{iter}{iteration number of the genetic algorithm.}
}
\value{
This function returns a list whose i-th element is a vector of
particle indices that inform the i-th particle, i.e., neighborhood of the
i-th particle.
}
\description{
Constructs a neighbourhood of each particle using 
particular topology.
}
\details{
If \code{method = "ring"} then each particle will have 
\code{par[1]} neighbours. By default \code{par[1] = 3}.
See section 3.2.1 of M. Clerc (2012) for 
additional details.
If \code{method = "wheel"} then there is a single (randomly selected) 
particle which informs (and is informed by) other particles, while there is 
no direct communication among other particles. 
If \code{method = "random"} then each particle randomly informs other
\code{par[1]} particles and itself. Note that duplicates are possible, so 
sometimes each particle may inform fewer than \code{par[1]} particles. 
By default \code{par[1] = 3}.
See section 3.2.2 of M. Clerc (2012) for more details.
If \code{method = "star"} then all particles are fully informed
by each other.
If \code{method = "random2"} then each particle will be self-informed
and is informed by the j-th particle with probability \code{par[1]}
(value between 0 and 1). By default \code{par[1] = 0.1}.
}
\examples{
# Prepare random number generator
set.seed(123)

# Ring topology with 5 neighbours
pso.nh(pop.n = 10, method = "ring", par = 5)

# Wheel topology
pso.nh(pop.n = 10, method = "wheel")

# Star topology
pso.nh(pop.n = 10, method = "star")

# Random topology where each particle 
# randomly informs 3 other particles
pso.nh(pop.n = 10, method = "random", par = 3)

# Random2 topology where each particle could
# be informed by another with probability 0.2
pso.nh(pop.n = 10, method = "random2", par = 0.2)

}
\references{
Maurice Clerc (2012). 
Standard Particle Swarm Optimisation.
\emph{HAL archive}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pso.R
\name{pso}
\alias{pso}
\title{Particle Swarm Optimization}
\usage{
pso(
  fn,
  gr = NULL,
  lower,
  upper,
  pop.n = 40,
  pop.initial = NULL,
  pop.method = "uniform",
  nh.method = "random",
  nh.par = 3,
  nh.adaptive = TRUE,
  velocity.method = "hypersphere",
  velocity.par = list(w = 1/(2 * log(2)), c1 = 0.5 + log(2), c2 = 0.5 + log(2)),
  hybrid.method = "rank",
  hybrid.par = 2,
  hybrid.prob = 0,
  hybrid.opt.par = NULL,
  hybrid.n = 1,
  constr.method = NULL,
  constr.par = NULL,
  random.order = TRUE,
  maxiter = 100,
  is.max = TRUE,
  info = TRUE,
  ...
)
}
\arguments{
\item{fn}{function to be maximized, i.e., fitness function.}

\item{gr}{gradient of the \code{fn}.}

\item{lower}{lower bound of the search space.}

\item{upper}{upper bound of the search space.}

\item{pop.n}{integer representing the size of the population.}

\item{pop.initial}{numeric matrix whose rows are particles to be
included in the initial population. A numeric vector will be coerced to
a single-row matrix.}

\item{pop.method}{the algorithm to be applied for a creation of 
the initial population. See 'Details' for additional information.}

\item{nh.method}{string representing the method (topology) to be used for
the creation of neighbourhoods. See 'Details' for additional information.}

\item{nh.par}{parameters of the topology algorithm.}

\item{nh.adaptive}{logical; if \code{TRUE} (default), then neighbourhoods
change every time the best known (to the swarm) fitness value has 
not increased. Neighbourhoods are updated according to the topology
defined via \code{nh.method} argument.}

\item{velocity.method}{string representing the method to be used for
the update of velocities.}

\item{velocity.par}{parameters of the velocity formula.}

\item{hybrid.method}{hybrid selection algorithm, i.e., mechanism 
determining which particles should be subject to local optimization.
See 'Details' for additional information.}

\item{hybrid.par}{parameters of the hybridization algorithm.}

\item{hybrid.prob}{probability of generating hybrids each iteration.}

\item{hybrid.opt.par}{parameters of the local optimization function
to be used for hybridization algorithm (including \code{fn} and \code{gr}).}

\item{hybrid.n}{number of hybrids that appear if hybridization
should take place during the iteration.}

\item{constr.method}{the algorithm to be applied for imposing constraints
on the particles. See 'Details' for additional information.}

\item{constr.par}{parameters of the constraint algorithm.}

\item{random.order}{logical; if \code{TRUE} (default), then 
particles-related routine will be implemented in a random order.}

\item{maxiter}{maximum number of iterations of the algorithm.}

\item{is.max}{logical; if \code{TRUE} (default) then fitness function
will be maximized. Otherwise it will be minimized.}

\item{info}{logical; if \code{TRUE} (default) then some optimization related 
information will be printed each iteration.}

\item{...}{additional parameters to be passed to 
\code{fn} and \code{gr} functions.}
}
\value{
This function returns an object of class \code{pso} that is a list
containing the following elements:
\itemize{
\item \code{par} - particle (solution) with the highest fitness
(objective function) value.
\item \code{value} - value of \code{fn} at \code{par}.
\item \code{population} - matrix of particles (solutions) from the 
last iteration of the algorithm.
\item \code{counts} - a two-element integer vector giving the number of
calls to \code{fn} and \code{gr} respectively.
\item \code{is.max} - identical to \code{is.max} input argument.
\item \code{fitness.history} - vector whose i-th element is the fitness
of the best particle in the i-th iteration.
\item \code{iter} - last iteration number.
}
}
\description{
This function allows the use of the particle swarm algorithm for
numeric global optimization of real-valued functions.
}
\details{
Default arguments have been set in accordance with SPSO 2011
algorithm proposed by M. Clerc (2012).

To find information on particular methods available via
the \code{pop.method}, \code{nh.method}, \code{velocity.method},
\code{hybrid.method}, and \code{constr.method}
arguments, please see the 'Details' section of 
\code{\link[gena]{gena.population}}, \code{\link[gena]{pso.nh}},
\code{\link[gena]{pso.velocity}}, \code{\link[gena]{gena.hybrid}}, 
and \code{\link[gena]{gena.constr}}, respectively.

It is possible to provide manually implemented functions for population
initialization, neighbourhood creation, velocity update, hybridization,
and constraints in a similar way as for \code{\link[gena]{gena}}.

By default, the function does not impose any constraints on the parameters.
If \code{constr.method = "bounds"}, then \code{lower} and \code{upper} 
constraints will be imposed. Lower bounds should be strictly smaller
than upper bounds.

Currently the only available termination condition is \code{maxiter}. We
are going to provide some additional termination conditions during
future updates.

Infinite values in \code{lower} and \code{upper} are substituted with
\code{-(.Machine$double.xmax * 0.9)} and \code{.Machine$double.xmax * 0.9},
respectively.

By default if \code{gr} is provided, then the BFGS algorithm will be used 
inside \code{\link[stats]{optim}} during hybridization.
Otherwise, \code{Nelder-Mead} will be used. 
Manual values for \code{\link[stats]{optim}} arguments may be provided 
(as a list) through \code{hybrid.opt.par} the argument.

For more information on particle swarm optimization, 
please see M. Clerc (2012).
}
\examples{
## Consider Ackley function
\donttest{
fn <- function(par, a = 20, b = 0.2)
{
  val <- a * exp(-b * sqrt(0.5 * (par[1] ^ 2 + par[2] ^ 2))) +
         exp(0.5 * (cos(2 * pi * par[1]) + cos(2 * pi * par[2]))) -
         exp(1) - a
  return(val)
}

# Maximize this function using particle swarm algorithm

set.seed(123)
lower <- c(-5, -100)
upper <- c(100, 5)
opt <- pso(fn = fn, 
           lower = lower, upper = upper,
           a = 20, b = 0.2)
print(opt$par)
}

## Consider Bukin function number 6

fn <- function(x, a = 20, b = 0.2)
{
  val <- 100 * sqrt(abs(x[2] - 0.01 * x[1] ^ 2)) + 0.01 * abs(x[1] + 10)
  return(val)
}

# Minimize this function using initially provided
# position for one of the particles
set.seed(777)
lower <- c(-15, -3)
upper <- c(-5, 3)
opt <- pso(fn = fn, 
           pop.initial = c(8, 2),
           lower = lower, upper = upper,
           is.max = FALSE)
print(opt$par)

}
\references{
M. Clerc (2012). 
Standard Particle Swarm Optimisation.
\emph{HAL archive}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/population.R
\name{gena.population}
\alias{gena.population}
\title{Population}
\usage{
gena.population(pop.n, lower, upper, pop.initial = NULL, method = "uniform")
}
\arguments{
\item{pop.n}{positive integer representing the number of chromosomes 
in population.}

\item{lower}{numeric vector whose i-th element determines the minimum
possible value for i-th gene.}

\item{upper}{numeric vector whose i-th element determines the maximum
possible value for i-th gene.}

\item{pop.initial}{numeric matrix whose rows are initial chromosomes
suggested by user.}

\item{method}{string representing the initialization method to be used.
For a list of possible values see Details.}
}
\value{
This function returns a matrix whose rows are chromosomes.
}
\description{
Initialize the population of chromosomes.
}
\details{
If \code{method = "uniform"}, then the i-th gene of each 
chromosome is randomly (uniformly) chosen between \code{lower[i]} and 
\code{upper[i]}. If \code{method = "normal"}, then the i-th gene is 
generated from a truncated normal distribution with the mean 
\code{(upper[i] + lower[i]) / 2} and standard deviation 
\code{(upper[i] - lower[i]) / 6}, where \code{lower[i]} and \code{upper[i]} 
are the lower and upper truncation bounds, respectively.
If \code{method = "hypersphere"}, then the population is simulated uniformly
from the hypersphere with center \code{upper - lower} and radius
\code{sqrt(sum((upper - lower) ^ 2))} via the 
\code{\link[gena]{rhypersphere}} function, setting \code{type = "inside"}.
}
\examples{
set.seed(123)
gena.population(pop.n = 10,
                lower = c(-1, -2, -3),
                upper = c(1, 0, -1),
                pop.initial = rbind(c(0, -1, -2),
                                    c(0.1, -1.2, -2.3)),
                method = "normal")
}
\references{
B. Kazimipour, X. Li, A. Qin (2014). 
A review of population initialization techniques for evolutionary algorithms.
\emph{2014 IEEE Congress on Evolutionary Computation}, 2585-2592,
<doi:10.1109/CEC.2014.6900618>.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gena.R
\name{gena}
\alias{gena}
\title{Genetic Algorithm}
\usage{
gena(
  fn,
  gr = NULL,
  lower,
  upper,
  pop.n = 100,
  pop.initial = NULL,
  pop.method = "uniform",
  mating.method = "rank",
  mating.par = NULL,
  mating.self = FALSE,
  crossover.method = "local",
  crossover.par = NULL,
  crossover.prob = 0.8,
  mutation.method = "constant",
  mutation.par = NULL,
  mutation.prob = 0.2,
  mutation.genes.prob = 1/length(lower),
  elite.n = min(10, 2 * round(pop.n/20)),
  elite.duplicates = FALSE,
  hybrid.method = "rank",
  hybrid.par = 2,
  hybrid.prob = 0,
  hybrid.opt.par = NULL,
  hybrid.n = 1,
  constr.method = NULL,
  constr.par = NULL,
  maxiter = 100,
  is.max = TRUE,
  info = TRUE,
  ...
)
}
\arguments{
\item{fn}{function to be maximized i.e. fitness function.}

\item{gr}{gradient of the \code{fn}.}

\item{lower}{lower bound of the search space.}

\item{upper}{upper bound of the search space.}

\item{pop.n}{integer representing the size of the population.}

\item{pop.initial}{numeric matrix whose rows are chromosomes to be
included in the initial population. A numeric vector will be coerced to
a single-row matrix.}

\item{pop.method}{the algorithm to be applied for the creation of 
the initial population. See 'Details' for additional information.}

\item{mating.method}{the algorithm to be applied for mating, i.e., selection
of parents. See 'Details' for additional information.}

\item{mating.par}{parameters of the mating (selection) algorithm.}

\item{mating.self}{logical; if \code{TRUE} then the chromosome may
mate with itself, i.e., both parents may be the same chromosome.}

\item{crossover.method}{an algorithm to be applied for crossover, i.e.,
creation of the children. See 'Details' for additional information.}

\item{crossover.par}{parameters of the crossover algorithm.}

\item{crossover.prob}{probability of the crossover for each pair of parents.}

\item{mutation.method}{algorithm to be applied for mutation, i.e., random
change in some genes of the children. 
See 'Details' for additional information.}

\item{mutation.par}{parameters of the mutation algorithm.}

\item{mutation.prob}{mutation probability for the chromosomes.}

\item{mutation.genes.prob}{mutation probability for the genes.}

\item{elite.n}{number of elite children, i.e., those that have the
highest function value and will be preserved for the next population.}

\item{elite.duplicates}{logical; if \code{TRUE} then some elite children
may have the same genes.}

\item{hybrid.method}{hybrid selection algorithm, i.e., mechanism 
determining which chromosomes should be subject to local optimization.
See 'Details' for additional information.}

\item{hybrid.par}{parameters of the hybridization algorithm.}

\item{hybrid.prob}{probability of generating hybrids each iteration.}

\item{hybrid.opt.par}{parameters of the local optimization function
to be used for the hybridization algorithm 
(including \code{fn} and \code{gr}).}

\item{hybrid.n}{number of hybrids that appear if hybridization
takes place during the iteration.}

\item{constr.method}{the algorithm to be applied for imposing constraints
on the chromosomes. See 'Details' for additional information.}

\item{constr.par}{parameters of the constraint algorithm.}

\item{maxiter}{maximum number of iterations of the algorithm.}

\item{is.max}{logical; if \code{TRUE} (default) then fitness function
will be maximized. Otherwise it will be minimized.}

\item{info}{logical; if \code{TRUE} (default) then some optimization-related 
information will be printed each iteration.}

\item{...}{additional parameters to be passed to 
\code{fn} and \code{gr} functions.}
}
\value{
This function returns an object of class \code{gena} that is a list
containing the following elements:
\itemize{
\item \code{par} - chromosome (solution) with the highest fitness
(objective function) value.
\item \code{value} - value of \code{fn} at \code{par}.
\item \code{population} - matrix of chromosomes (solutions) from the 
last iteration of the algorithm.
\item \code{counts} - a two-element integer vector giving the number of
calls to \code{fn} and \code{gr} respectively.
\item \code{is.max} - identical to \code{is.max} input argument.
\item \code{fitness.history} - vector whose i-th element is the fitness
of the best chromosome in the i-th iteration.
\item \code{iter} - last iteration number.
}
}
\description{
This function allows the use of a genetic algorithm for
numeric global optimization of real-valued functions.
}
\details{
To find information on particular methods available via the 
\code{pop.method},
\code{mating.method}, \code{crossover.method}, \code{mutation.method},
\code{hybrid.method}, and \code{constr.method}
arguments, please see the 'Details' section of 
\code{\link[gena]{gena.population}}, \code{\link[gena]{gena.crossover}},
\code{\link[gena]{gena.mutation}}, \code{\link[gena]{gena.hybrid}}, 
and \code{\link[gena]{gena.constr}}, respectively. For example to
find information on possible values of \code{mutation.method} and
\code{mutation.par} arguments see description of \code{method} and
\code{par} arguments of \code{\link[gena]{gena.mutation}} function.

It is possible to provide manually implemented functions for 
population initialization, mating, crossover, mutation, and hybridization. 
For example, a manual mutation function may be provided through
the \code{mutation.method} argument. It should have the same signature 
(arguments) as the \code{\link[gena]{gena.mutation}} function and return 
the same object, i.e., the matrix of chromosomes of the appropriate size.
Manually implemented functions for other operators (crossover, mating,
and so on) may be provided in a similar way.

By default, the function does not impose any constraints on the parameters.
If \code{constr.method = "bounds"}, then \code{lower} and \code{upper} 
constraints will be imposed. Lower bounds should be strictly smaller
than upper bounds.

Currently the only available termination condition is \code{maxiter}. We
are going to provide some additional termination conditions during
future updates.

Infinite values in \code{lower} and \code{upper} are substituted with
\code{-(.Machine$double.xmax * 0.9)} and \code{.Machine$double.xmax * 0.9},
respectively.

By default, if \code{gr} is provided, then the BFGS algorithm will be used 
inside \code{\link[stats]{optim}} during hybridization.
Otherwise, \code{Nelder-Mead} will be used. 
Manual values for \code{\link[stats]{optim}} arguments may be provided 
(as a list) through the \code{hybrid.opt.par} argument.

Arguments \code{pop.n} and \code{elite.n} should be even integers, and
\code{elite.n} should be greater than 2. If these arguments are odd integers, 
then they will be coerced to even integers by adding 1. 
Also, \code{pop.n} should be greater than \code{elite.n} by at least 2.

For more information on the genetic algorithm,
please see Katoch et al. (2020).
}
\examples{
## Consider Ackley function
\donttest{
fn <- function(par, a = 20, b = 0.2)
{
  val <- a * exp(-b * sqrt(0.5 * (par[1] ^ 2 + par[2] ^ 2))) +
         exp(0.5 * (cos(2 * pi * par[1]) + cos(2 * pi * par[2]))) -
         exp(1) - a
  return(val)
}

# Maximize this function using classical
# genetic algorithm setup
set.seed(123)
lower <- c(-5, -100)
upper <- c(100, 5)
opt <- gena(fn = fn, 
            lower = lower, upper = upper,
            hybrid.prob = 0,
            a = 20, b = 0.2)
print(opt$par)

# Replicate optimization using hybridization
opt <- gena(fn = fn, 
            lower = lower, upper = upper,
            hybrid.prob = 0.2,
            a = 20, b = 0.2)
print(opt$par)
}

## Consider Rosenbrock function
fn <- function(par, a = 100)
{
  val <- -(a * (par[2] - par[1] ^ 2) ^ 2 + (1 - par[1]) ^ 2 +
           a * (par[3] - par[2] ^ 2) ^ 2 + (1 - par[2]) ^ 2)
  return(val)
}

# Apply genetic algorithm
lower <- rep(-10, 3)
upper <- rep(10, 3)
set.seed(123)
opt <- gena(fn = fn,
            lower = lower, upper = upper,
            a = 100)
print(opt$par)

\donttest{
# Improve the results by hybridization
opt <- gena(fn = fn,
            lower = lower, upper = upper,
            hybrid.prob = 0.2,
            a = 100)
print(opt$par)
}

# Provide manually implemented mutation function
# which simply randomly sorts genes. 
# Note that this function should have the same 
# arguments as gena.mutation.
mutation.my <- function(children, lower, upper, 
                        prob, prob.genes, 
                        method, par, iter)
{
  # Get dimensional data
  children.n <- nrow(children)
  genes.n <- ncol(children)
  
  # Select chromosomes that should mutate
  random_values <- runif(children.n, 0, 1)
  mutation_ind <- which(random_values <= prob)
   
  # Mutate chromosomes by randomly sorting
  # their genes
  for (i in mutation_ind)
  {
    children[i, ] <- children[i, sample(1:genes.n)]
  }
  
  # Return mutated chromosomes
  return(children)
}

opt <- gena(fn = fn,
            lower = lower, upper = upper,
            mutation.method = mutation.my,
            a = 100)
print(opt$par)

}
\references{
S. Katoch, S. Chauhan, V. Kumar (2020). 
A review on genetic algorithm: past, present, and future.
\emph{Multimedia Tools and Applications}, 80, 8091-8126.
<doi:10.1007/s11042-020-10139-6>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hybrid.R
\name{gena.hybrid}
\alias{gena.hybrid}
\title{Hybridization}
\usage{
gena.hybrid(
  population,
  fitness,
  hybrid.n = 1,
  method,
  par,
  opt.par,
  info = FALSE,
  iter = NULL,
  ...
)
}
\arguments{
\item{population}{numeric matrix whose rows are chromosomes, i.e., vectors of 
parameter values.}

\item{fitness}{numeric vector whose \code{i}-th element is the value of 
\code{fn} at point \code{population[i, ]}.}

\item{hybrid.n}{positive integer representing the number of hybrids.}

\item{method}{hybridization method to improve chromosomes via local search.}

\item{par}{additional parameters to be passed depending on the \code{method}.}

\item{opt.par}{parameters of the local optimization function
to be used for hybridization algorithm (including \code{fn} and \code{gr}).}

\item{info}{logical; if \code{TRUE} then some optimization-related 
information will be printed each iteration.}

\item{iter}{iteration number of the genetic algorithm.}

\item{...}{additional parameters to be passed to 
\code{fn} and \code{gr} functions.}
}
\value{
The function returns a list with the following elements:
\itemize{
\item \code{population} - matrix whose rows are chromosomes including hybrids.
\item \code{fitness} - vector whose i-th element is the fitness of the
i-th chromosome.
\item \code{hybrids.ind} - vector of indexes of chromosomes selected for
hybridization.
\item \code{counts} - a two-element integer vector giving the number of
calls to \code{fn} and \code{gr} respectively.
}
}
\description{
Hybridization method (algorithm) to be used in the
genetic algorithm.
}
\details{
This function uses \code{\link[gena]{gena.mating}} function to 
select hybrids. Therefore \code{method} and \code{par} arguments will
be passed to this function. If some chromosomes selected to become hybrids
are duplicated, then these duplicates will not be subject to local
optimization, i.e., the number of hybrids will be decreased by the number
of duplicates (actual number of hybrids during some iterations may be 
lower than \code{hybrid.n}).

Currently \code{\link[stats]{optim}} is the only available local
optimizer. Therefore \code{opt.par} is a list containing parameters
that should be passed to \code{\link[stats]{optim}}.

For more information on hybridization, please see El-Mihoub et al. (2006).
}
\examples{
# Consider the following fitness function
fn <- function(x)
{
  val <- x[1] * x[2] - x[1] ^ 2 - x[2] ^ 2
}

# Also let's provide its gradient (optional)
gr <- function(x)
{
  val <- c(x[2] - 2 * x[1],
           x[1] - 2 * x[2])
}

# Randomly initialize the population
set.seed(123)
n_population <- 10
population <- gena.population(pop.n = n_population,
                              lower = c(-5, -5), 
                              upper = c(5, 5))

# Calculate fitness of each chromosome
fitness <- rep(NA, n_population)
for(i in 1:n_population)
{
  fitness[i] <- fn(population[i, ])
}

# Perform hybridization
hybrids <- gena.hybrid(population = population,
                       fitness = fitness,
                       opt.par = list(fn = fn,
                                      gr = gr,
                                      method = "BFGS",
                                      control = list(fnscale = -1,
                                                     abstol = 1e-10,
                                                     reltol = 1e-10,
                                                     maxit = 1000)),
                       hybrid.n = 2,
                       method = "rank",
                       par = 0.8)
print(hybrids)

}
\references{
T. El-Mihoub, A. Hopgood, L. Nolle, B. Alan (2006). 
Hybrid Genetic Algorithms: A Review.
\emph{Engineering Letters}, 13 (3), 124-137.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pso.R
\name{print.summary.pso}
\alias{print.summary.pso}
\title{Summary for "pso" object}
\usage{
\method{print}{summary.pso}(x, ...)
}
\arguments{
\item{x}{Object of class "pso"}

\item{...}{further arguments (currently ignored)}
}
\value{
This function returns \code{x} input argument.
}
\description{
Summary for "pso" object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mutation.R
\name{gena.mutation}
\alias{gena.mutation}
\title{Mutation}
\usage{
gena.mutation(
  children,
  lower,
  upper,
  prob = 0.2,
  prob.genes = 1/nrow(children),
  method = "constant",
  par = 1,
  iter = NULL
)
}
\arguments{
\item{children}{numeric matrix whose rows are children, i.e., vectors of 
parameter values.}

\item{lower}{lower bound of the search space.}

\item{upper}{upper bound of the search space.}

\item{prob}{probability of mutation for a child.}

\item{prob.genes}{numeric vector or numeric value representing the
probability of mutation of a child's gene. See 'Details'.}

\item{method}{mutation method to be used for transforming genes of children.}

\item{par}{additional parameters to be passed depending on the \code{method}.}

\item{iter}{iteration number of the genetic algorithm.}
}
\value{
The function returns a matrix whose rows are children
(after mutation has been applied to some of them).
}
\description{
Mutation method (algorithm) to be used in the
genetic algorithm.
}
\details{
Denote \code{children} by \eqn{C^{child}} whose \code{i}-th row 
\code{children[i, ]} is a chromosome \eqn{c_{i}^{child}}, i.e., the vector of 
parameter values of the function being optimized \eqn{f(.)} that is
provided via the \code{fn} argument of \code{\link[gena]{gena}}.
The elements of chromosome \eqn{c_{ij}^{child}} are genes 
representing parameters values.

Mutation algorithm determines random transformation of children's genes. 
Each child may be selected for mutation with probability \code{prob}.
If \eqn{i}-th child is selected for mutation and \code{prob.genes} is a 
vector then \eqn{j}-th gene of this child
is transformed with probability \code{prob.genes[j]}. If \code{prob.genes}
is a constant then this probability is the same for all genes.

Argument \code{method} determines the particular mutation algorithm to 
be applied. Denote by \eqn{\tau} the vector of parameters used by the 
algorithm. Note that \eqn{\tau} corresponds to \code{par}.
Also let us denote by \eqn{c_{ij}^{mutant}} the value of 
gene \eqn{c_{ij}^{child}} after mutation.

If \code{method = "constant"}, then \eqn{c_{ij}^{mutant}}
is a uniform random variable between \code{lower[j]} and \code{upper[j]}.

If \code{method = "normal"}, then \eqn{c_{ij}^{mutant}}
equals the sum of \eqn{c_{ij}^{child}} and a normal random variable
with zero mean and standard deviation \code{par[j]}. 
By default, \code{par} is a vector of ones of length \code{ncol(children)},
so \code{par[j] = 1} for all \code{j}.

If \code{method = "percent"}, then \eqn{c_{ij}^{mutant}} is generated 
from \eqn{c_{ij}^{child}}  by equiprobably increasing or decreasing it 
by \eqn{q} percent,
where \eqn{q} is a uniform random variable between \eqn{0} and \code{par[j]}. 
Note that \code{par} may also be a constant; then all
genes have the same maximum possible percentage change.
By default, \code{par = 20}.

For more information on mutation algorithms,
please see Patil and Bhende (2014).
}
\examples{
# Randomly initialize some children
set.seed(123)
children.n <- 10
children <- gena.population(pop.n = children.n,
                            lower = c(-5, -5), 
                            upper = c(5, 5))
                           
# Perform the mutation
mutants <- gena.mutation(children = children,
                         prob = 0.6,
                         prob.genes = c(0.7, 0.8),
                         par = 30,
                         method = "percent")
print(mutants)
}
\references{
S. Patil, M. Bhende. (2014). 
Comparison and Analysis of Different Mutation Strategies to improve the 
Performance of Genetic Algorithm.
\emph{International Journal of Computer Science and 
Information Technologies}, 5 (3), 4669-4673.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gena.R
\name{summary.gena}
\alias{summary.gena}
\title{Summarizing gena Fits}
\usage{
\method{summary}{gena}(object, ...)
}
\arguments{
\item{object}{Object of class "gena"}

\item{...}{further arguments (currently ignored)}
}
\value{
This function returns the same list as \code{\link[gena]{gena}} 
function changing its class to "summary.gena".
}
\description{
Summarizing gena Fits
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pso.R
\name{plot.pso}
\alias{plot.pso}
\title{Plot best found fitnesses during genetic algorithm}
\usage{
\method{plot}{pso}(x, y = NULL, ...)
}
\arguments{
\item{x}{Object of class "pso"}

\item{y}{this parameter currently ignored}

\item{...}{further arguments (currently ignored)}
}
\value{
This function does not return anything.
}
\description{
Plot best found fitnesses during genetic algorithm
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gena.R
\name{print.summary.gena}
\alias{print.summary.gena}
\title{Summary for "gena" object}
\usage{
\method{print}{summary.gena}(x, ...)
}
\arguments{
\item{x}{Object of class "gena"}

\item{...}{further arguments (currently ignored)}
}
\value{
This function returns \code{x} input argument.
}
\description{
Summary for "gena" object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crossover.R
\name{gena.crossover}
\alias{gena.crossover}
\title{Crossover}
\usage{
gena.crossover(
  parents,
  fitness = NULL,
  prob = 0.8,
  method = "local",
  par = NULL,
  iter = NULL
)
}
\arguments{
\item{parents}{numeric matrix whose rows are parents, i.e., vectors of 
parameters values.}

\item{fitness}{numeric vector whose \code{i}-th element is the value of 
\code{fn} at point \code{population[i, ]}.}

\item{prob}{probability of crossover.}

\item{method}{crossover method to be used for making children.}

\item{par}{additional parameters to be passed depending on the \code{method}.}

\item{iter}{iteration number of the genetic algorithm.}
}
\value{
The function returns a matrix whose rows are children.
}
\description{
Crossover method (algorithm) to be used in the
genetic algorithm.
}
\details{
Denote \code{parents} by \eqn{C^{parent}} whose \code{i}-th row 
\code{parents[i, ]} is a chromosome \eqn{c_{i}^{parent}}, i.e., the vector of 
parameter values of the function being optimized \eqn{f(.)} that is
provided via the \code{fn} argument of \code{\link[gena]{gena}}.
The elements of chromosome \eqn{c_{ij}^{parent}} are genes 
representing parameters values.

Crossover algorithm determines the way parents produce children. 
During crossover, each of the randomly selected pairs of parents 
\eqn{c_{i}^{parent}}, \eqn{c_{i + 1}^{parent}}
produces two children 
\eqn{c_{i}^{child}}, \eqn{c_{i + 1}^{child}}, 
where \eqn{i} is odd. Each pair of parents is selected with 
probability \code{prob}. If a pair of parents has not been selected
for crossover, then corresponding children and parents coincide, i.e., 
\eqn{c_{i}^{child}=c_{i}^{parent}} and
\eqn{c_{i+1}^{child}=c_{i+1}^{parent}}.

Argument \code{method} determines particular crossover algorithm to 
be applied. Denote by \eqn{\tau} the vector of parameters used by the 
algorithm. Note that \eqn{\tau} corresponds to \code{par}.

If \code{method = "split"} then each gene of the first child will
be equiprobably picked from the first or from the second parent. So 
\eqn{c_{ij}^{child}} may be equal to \eqn{c_{ij}^{parent}}
or \eqn{c_{(i+1)j}^{parent}} with equal probability. The second
child is the reversal of the first one in the sense that if the first child 
gets a particular gene of the first (second) parent, then the second child 
gets this gene from the second (first) parent, i.e., if
\eqn{c_{ij}^{child}=c_{ij}^{parent}} then 
\eqn{c_{(i+1)j}^{child}=c_{(i+1)j}^{parent}}; if 
\eqn{c_{ij}^{child}=c_{(i+1)j}^{parent}} then 
\eqn{c_{(i+1)j}^{child}=c_{ij}^{parent}}.

If \code{method = "arithmetic"} then:
\deqn{c_{i}^{child}=\tau_{1}c_{i}^{parent}+
\left(1-\tau_{1}\right)c_{i+1}^{parent}}
\deqn{c_{i+1}^{child}=\left(1-\tau_{1}\right)c_{i}^{parent}+
\tau_{1}c_{i+1}^{parent}}
where \eqn{\tau_{1}} is \code{par[1]}. By default \code{par[1] = 0.5}.

If \code{method = "local"} then the procedure is the same as 
for "arithmetic" method but \eqn{\tau_{1}} is a uniform random
value between 0 and 1.

If \code{method = "flat"} then \eqn{c_{ij}^{child}} is a uniform
random number between \eqn{c_{ij}^{parent}} and 
\eqn{c_{(i+1)j}^{parent}}. 
Similarly for the second child \eqn{c_{(i+1)j}^{child}}.

For more information on crossover algorithms,
please see Kora and Yadlapalli (2017).
}
\examples{
# Randomly initialize the parents
set.seed(123)
parents.n <- 10
parents <- gena.population(pop.n = parents.n,
                           lower = c(-5, -5), 
                           upper = c(5, 5))
                           
# Perform the crossover
children <- gena.crossover(parents = parents,
                           prob = 0.6,
                           method = "local")
print(children)
}
\references{
P. Kora, P. Yadlapalli. (2017). 
Crossover Operators in Genetic Algorithms: A Review.
\emph{International Journal of Computer Applications}, 162 (10), 34-36,
<doi:10.5120/ijca2017913370>.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/local.R
\name{genaDiff}
\alias{genaDiff}
\alias{gena.grad}
\alias{gena.hessian}
\title{Numeric Differentiation}
\usage{
gena.grad(
  fn,
  par,
  eps = sqrt(.Machine$double.eps) * abs(par),
  method = "central-difference",
  fn.args = NULL
)

gena.hessian(
  fn = NULL,
  gr = NULL,
  par,
  eps = sqrt(.Machine$double.eps) * abs(par),
  fn.args = NULL,
  gr.args = NULL,
  is.gc = FALSE
)
}
\arguments{
\item{fn}{function for which gradient or Hessian should be calculated.}

\item{par}{point (parameter values) at which \code{fn} should be 
differentiated.}

\item{eps}{numeric vector representing the increment of the \code{par}. 
So \code{eps[i]} represents the increment of \code{par[i]}. If \code{eps} is
a constant, then all increments are the same.}

\item{method}{numeric differentiation method: "central-difference" or
"forward-difference".}

\item{fn.args}{list containing arguments of \code{fn} except \code{par}.}

\item{gr}{gradient function of \code{fn}.}

\item{gr.args}{list containing arguments of \code{gr} except \code{par}.}

\item{is.gc}{logical; if \code{TRUE}, then garbage collection is called 
inside loops to reduce memory usage.}
}
\value{
Function \code{gena.grad} returns a vector that is the gradient of 
\code{fn} at the point \code{par} calculated via the \code{method} numeric 
differentiation approach using the increment \code{eps}.

Function \code{gena.hessian} returns a matrix that is the Hessian of 
\code{fn} at the point \code{par}.
}
\description{
Numeric estimation of the gradient and Hessian.
}
\details{
It is possible to substantially improve numeric Hessian accuracy 
by using analytical gradient \code{gr}. If both \code{fn} and \code{gr}
are provided then only \code{gr} will be used. If only \code{fn} is provided 
for \code{gena.hessian} then \code{eps} will be transformed to 
\code{sqrt(eps)} for numeric stability purposes.
}
\examples{
# Consider the following function
fn <- function(par, a = 1, b = 2)
{
  val <- par[1] * par[2] - a * par[1] ^ 2 - b * par[2] ^ 2
}

# Calculate the gradient at point (2, 5) with respect to 'par' 
# when 'a = 1' and 'b = 1'
par <- c(2, 5)
fn.args = list(a = 1, b = 1)
gena.grad(fn = fn, par = par, fn.args = fn.args)

# Calculate Hessian at the same point
gena.hessian(fn = fn, par = par, fn.args = fn.args)

# Repeat calculation of the Hessian using analytical gradient
gr <- function(par, a = 1, b = 2)
{
  val <- c(par[2] - 2 * a * par[1],
           par[1] - 2 * b * par[2])
}
gena.hessian(gr = gr, par = par, gr.args = fn.args)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mating.R
\name{gena.mating}
\alias{gena.mating}
\title{Mating}
\usage{
gena.mating(
  population,
  fitness,
  parents.n,
  method = "rank",
  par = NULL,
  self = FALSE,
  iter = NULL
)
}
\arguments{
\item{population}{numeric matrix whose rows are chromosomes, i.e., vectors of 
parameter values.}

\item{fitness}{numeric vector whose \code{i}-th element is the value of 
\code{fn} at point \code{population[i, ]}.}

\item{parents.n}{even positive integer representing the number of parents.}

\item{method}{mating method to be used for selection of parents.}

\item{par}{additional parameters to be passed depending on the \code{method}.}

\item{self}{logical; if \code{TRUE} then a chromosome may mate with itself. 
Otherwise, mating is allowed only between different chromosomes.}

\item{iter}{iteration number of the genetic algorithm.}
}
\value{
The function returns a list with the following elements:
\itemize{
\item \code{parents} - matrix whose rows are parents. The number of
rows of this matrix equals to \code{parents.n} while the number of columns
is \code{ncol(population)}.
\item \code{fitness} - vector whose i-th element is the fitness of the
i-th parent.
\item \code{ind} - vector whose i-th element is the index of the i-th
parent in the population, so \code{$parents[i, ]} equals 
\code{population[ind[i], ]}.
}
}
\description{
Mating (selection) method (algorithm) to be used in the
genetic algorithm.
}
\details{
Denote \code{population} by \eqn{C} whose \code{i}-th row 
\code{population[i, ]} is a chromosome \eqn{c_{i}}, i.e., the vector of 
parameter values of the function being optimized \eqn{f(.)} that is
provided via the \code{fn} argument of \code{\link[gena]{gena}}.
The elements of chromosome \eqn{c_{ij}} are genes representing parameters 
values. Argument \code{fitness} is a vector of function values at
corresponding chromosomes i.e. \code{fitness[i]} corresponds to
\eqn{f_{i}=f(c_{i})}. Total number of chromosomes in population
\eqn{n_{population}} equals to \code{nrow(population)}.

The mating algorithm determines the selection of chromosomes that will become 
parents. During each iteration of mating, one of the chromosomes becomes a 
parent until there are \eqn{n_{parents}} (i.e., \code{parents.n}) parents 
selected. Each chromosome may become a parent multiple times or not become a 
parent at all.

Denote by \eqn{c^{s}_{i}} the \eqn{i}-th selected parent. Parents
\eqn{c^{s}_{i}} and \eqn{c^{s}_{i + 1}} form a pair that will further
produce a child (offspring), where \eqn{i} is odd.
If \code{self = FALSE}, then for each pair of parents 
\eqn{(c_{i}^s, c_{i+1}^s)} it is ensured that 
\eqn{c^{s}_{i} \ne c^{s}_{i + 1}} except in the case when there are several 
identical chromosomes in the population. However, \code{self} is ignored 
if \code{method} is \code{"tournament"}, so in this case self-mating
is always possible.

Denote by \eqn{p_{i}} the probability of a chromosome becoming a parent.
Note that each chromosome may become a parent multiple times.
The probability \eqn{p_{i}\left(f_{i}\right)} is a function 
of fitness \eqn{f_{i}}. Usually this function is non-decreasing, so 
fitter chromosomes have a higher probability of becoming a parent. 
There is also an intermediate value \eqn{w_{i}} called the weight such that:
\deqn{p_{i}=\frac{w_{i}}{\sum\limits_{j=1}^{n_{population}}w_{j}}}
Therefore, all weights \eqn{w_{i}} are proportional to the corresponding 
probabilities \eqn{p_{i}} by the same factor (the sum of weights).

Argument \code{method} determines particular mating algorithm to be applied.
Denote by \eqn{\tau} the vector of parameters used by the algorithm.
Note that \eqn{\tau} corresponds to \code{par}. The algorithm determines
a particular form of the \eqn{w_{i}\left(f_{i}\right)} function which 
in turn determines \eqn{p_{i}\left(f_{i}\right)}.

If \code{method = "constant"}, then all weights and probabilities are equal:
\deqn{w_{i}=1 => p_{i}=\frac{1}{n_{population}}}

If \code{method = "rank"}, then each chromosome receives a rank \eqn{r_{i}} 
based on the fitness \eqn{f_{i}} value. So if \code{j}-th chromosome is the
fittest one and \code{k}-th chromosome has the lowest fitness value, then
\eqn{r_{j}=n_{population}} and  \eqn{r_{k}=1}. The relationship
between weight \eqn{w_{i}} and rank \eqn{r_{i}} is as follows:
\deqn{w_{i}=\left(\frac{r_{i}}{n_{population}}\right)^{\tau_{1}}}
The greater value of \eqn{\tau_{1}} the greater portion of probability will
be delivered to fitter chromosomes. 
Default value is \eqn{\tau_{1} = 0.5} so \code{par = 0.5}.

If \code{method = "fitness"} then weights are calculated as follows:
\deqn{w_{i}=\left(f_{i} - 
                  \min\left(f_{1},...,f_{n_{population}}\right) + 
                  \tau_{1}\right)^{\tau_{2}}}
By default \eqn{\tau_{1}=10} and \eqn{\tau_{2}=0.5}, i.e., 
\code{par = c(10, 0.5)}. There is a restriction \eqn{\tau_{1}\geq0}
ensuring that the expression in brackets is non-negative.

If \code{method = "tournament"} then  \eqn{\tau_{1}} (i.e., \code{par}) 
chromosomes will be randomly selected with equal probabilities and without 
replacement. Then the chromosome with the highest fitness 
(among these selected chromosomes) will become a parent.
It is possible to provide the representation of this algorithm via 
probabilities \eqn{p_{i}} but the formulas are numerically unstable.
By default \code{par = min(5, ceiling(parents.n * 0.1))}.

Validation and default values assignment for \code{par} are performed inside
the \code{\link[gena]{gena}} function not in \code{\link[gena]{gena.mating}}.
This allows validation to be performed a single time instead of repeating it
each iteration of genetic algorithm.

For more information on mating (selection) algorithms,
please see Shukla et al. (2015).
}
\examples{
# Consider the following fitness function
fn <- function(x)
{
  val <- x[1] * x[2] - x[1] ^ 2 - x[2] ^ 2
}

# Randomly initialize the population
set.seed(123)
n.pop <- 10
population <- gena.population(pop.n = n.pop,
                              lower = c(-5, -5), 
                              upper = c(5, 5))

# Calculate fitness of each chromosome
fitness <- rep(NA, n.pop)
for(i in 1:n.pop)
{
  fitness[i] <- fn(population[i, ])
}

# Perform mating to select parents
parents <- gena.mating(population = population,
                       fitness = fitness,
                       parents.n = n.pop,
                       method = "rank",
                       par = 0.8)
print(parents)

}
\references{
A. Shukla, H. Pandey, D. Mehrotra (2015). 
Comparative review of selection techniques in genetic algorithm.
\emph{2015 International Conference on Futuristic Trends on Computational 
Analysis and Knowledge Management (ABLAZE)}, 515-519,
<doi:10.1109/ABLAZE.2015.7154916>.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constr.R
\name{gena.constr}
\alias{gena.constr}
\title{Constraints}
\usage{
gena.constr(population, method = "bounds", par, iter)
}
\arguments{
\item{population}{numeric matrix which rows are chromosomes i.e. vectors of 
parameters values.}

\item{method}{method used to impose constraints.}

\item{par}{additional parameters to be passed depending on the \code{method}.}

\item{iter}{iteration number of the genetic algorithm.}
}
\value{
The function returns a list with the following elements:
\itemize{
\item \code{population} - matrix which rows are chromosomes after
constraints have been imposed.
\item \code{constr.ind} - matrix of logical values which (i, j)-th
elements equals \code{TRUE} (\code{FALSE} otherwise) if j-th jene of
i-th chromosome is a subject to constraint.
}
}
\description{
Impose constraints on chromosomes.
}
\details{
If \code{method = "bounds"} then chromosomes will be bounded
between \code{par$lower} and \code{par$upper}.
}
\examples{
# Randomly initialize population
set.seed(123)
population <- gena.population(pop.n = 10,
                              lower = c(-5, -5), 
                              upper = c(5, 5))
                           
# Impose lower and upper bounds constraints
pop.constr <- gena.constr(population, 
                          method = "bounds",
                          par = list(lower = c(-1, 2),
                                     upper = c(1, 5)))
print(pop.constr)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/velocity.R
\name{pso.velocity}
\alias{pso.velocity}
\title{Velocity}
\usage{
pso.velocity(
  population,
  method = "hypersphere",
  par = list(w = 1/(2 * log(2)), c1 = 0.5 + log(2), c2 = 0.5 + log(2)),
  velocity,
  best.pn,
  best.nh,
  best.pn.fitness,
  best.nh.fitness,
  iter = 1
)
}
\arguments{
\item{population}{numeric matrix whose rows are particles, i.e., vectors of 
parameters values.}

\item{method}{string representing method to be used for velocities 
calculation. See 'Details' for additional information.}

\item{par}{additional parameters to be passed depending on the \code{method}.}

\item{velocity}{matrix whose i-th row is a velocity of the i-th particle.}

\item{best.pn}{numeric matrix whose i-th row is a best personal position 
known by the i-th particle.}

\item{best.nh}{numeric matrix whose i-th row is a best personal position 
in a neighbourhood of the i-th particle.}

\item{best.pn.fitness}{numeric vector whose i-th element is the value of
the fitness function at point \code{best.pn[i, ]}.}

\item{best.nh.fitness}{numeric vector whose i-th element is the value of
the fitness function at point \code{best.nh[i, ]}.}

\item{iter}{iteration number of the genetic algorithm.}
}
\value{
This function returns a matrix whose i-th row represents the 
updated velocity of the i-th particle.
}
\description{
Calculates (updates) velocities of the particles.
}
\details{
If \code{method = "classic"}, then the classical velocity formula
is used:
\deqn{v_{i,j,(t+1)}=w\times v_{i,j,t} + 
                     c_{1}\times u_{1,i,j} \times b^{pn}_{i,j,t} + 
                     c_{2}\times u_{2,i,j} \times b^{nh}_{i,j,t}}
                     
 where \eqn{v_{i, j, t}} is the velocity of the \eqn{i}-th particle 
 with respect to the \eqn{j}-th component at time \eqn{t}. 
 The random variables \eqn{u_{1,i,j}} and \eqn{u_{2,i,j}} are i.i.d. with 
 respect to all indices and follow the standard uniform distribution 
 \eqn{U(0, 1)}. The variable \eqn{b^{pn}_{i,j,t}} is the \eqn{j}-th 
 component of the best known personal position of the particle up to time 
 period \eqn{t}. Similarly, \eqn{b^{nh}_{i,j,t}} is the \eqn{j}-th component 
 of the best known position in the neighbourhood of the \eqn{i}-th particle.
 The hyperparameters \eqn{w}, \eqn{c_{1}}, and \eqn{c_{2}} may be provided
 via the \code{par} argument as a list with elements \code{par$w}, 
 \code{par$c1}, and \code{par$c2}, respectively. 
 
 If \code{method = "hypersphere"}, then rotation-invariant formula from
 sections 3.4.2 and 3.4.3 of M. Clerc (2012) is used, with arguments
 identical to the classical method. To simulate a random variate from
 the hypersphere, the function \code{\link[gena]{rhypersphere}} is used,
 setting \code{type = "non-uniform"}.
 
 In accordance with M. Clerc (2012), the 
 default values are \code{par$w = 1/(2 * log(2))},
 \code{par$c1 = 0.5 + log(2)}, and \code{par$c2 = 0.5 + log(2)}.
}
\references{
Maurice Clerc (2012). 
Standard Particle Swarm Optimisation.
\emph{HAL archive}.
}
